@@ -398,7 +398,8 @@ bool PlaylistDAO::removeTracksFromPlaylist(int playlistId, int startIndex) {
         return false;
     }
     transaction.commit();
-    emit tracksChanged(QSet<int>{playlistId});
+    emit playlistContentChanged(QSet<int>{playlistId});
+    emit tracksRemoved(QSet<int>{playlistId});
     return true;
 }
 
@@ -440,7 +441,8 @@ bool PlaylistDAO::appendTracksToPlaylist(const QList<TrackId>& trackIds, const i
         // TODO(XXX) don't emit if the track didn't add successfully.
         emit trackAdded(playlistId, trackId, insertPosition++);
     }
-    emit tracksChanged(QSet<int>{playlistId});
+    emit tracksAdded(QSet<int>{playlistId});
+    emit playlistContentChanged(QSet<int>{playlistId});
     return true;
 }
 
@@ -551,7 +553,7 @@ bool PlaylistDAO::isHidden(const int playlistId) const {
 
 void PlaylistDAO::removeHiddenTracks(const int playlistId) {
     ScopedTransaction transaction(m_database);
-    // This query deletes all tracks marked as deleted and all
+    // This query deletes all tracks marked as hidden and all
     // phantom track_ids with no match in the library table
     QSqlQuery query(m_database);
     query.prepare(QStringLiteral(
@@ -576,14 +578,16 @@ void PlaylistDAO::removeHiddenTracks(const int playlistId) {
     }
 
     transaction.commit();
-    emit tracksChanged(QSet<int>{playlistId});
+    emit playlistContentChanged(QSet<int>{playlistId});
+    emit tracksRemoved(QSet<int>{playlistId});
 }
 
 void PlaylistDAO::removeTracksFromPlaylistById(int playlistId, TrackId trackId) {
     ScopedTransaction transaction(m_database);
     removeTracksFromPlaylistByIdInner(playlistId, trackId);
     transaction.commit();
-    emit tracksChanged(QSet<int>{playlistId});
+    emit playlistContentChanged(QSet<int>{playlistId});
+    emit tracksRemoved(QSet<int>{playlistId});
 }
 
 void PlaylistDAO::removeTracksFromPlaylistByIdInner(int playlistId, TrackId trackId) {
@@ -612,7 +616,8 @@ void PlaylistDAO::removeTrackFromPlaylist(int playlistId, int position) {
     ScopedTransaction transaction(m_database);
     removeTracksFromPlaylistInner(playlistId, position);
     transaction.commit();
-    emit tracksChanged(QSet<int>{playlistId});
+    emit playlistContentChanged(QSet<int>{playlistId});
+    emit tracksRemoved(QSet<int>{playlistId});
 }
 
 void PlaylistDAO::removeTracksFromPlaylist(int playlistId, const QList<int>& positions) {
@@ -627,7 +632,8 @@ void PlaylistDAO::removeTracksFromPlaylist(int playlistId, const QList<int>& pos
         removeTracksFromPlaylistInner(playlistId, position);
     }
     transaction.commit();
-    emit tracksChanged(QSet<int>{playlistId});
+    emit playlistContentChanged(QSet<int>{playlistId});
+    emit tracksRemoved(QSet<int>{playlistId});
 }
 
 void PlaylistDAO::removeTracksFromPlaylistInner(int playlistId, int position) {
@@ -722,7 +728,8 @@ bool PlaylistDAO::insertTrackIntoPlaylist(TrackId trackId, const int playlistId,
 
     m_playlistsTrackIsIn.insert(trackId, playlistId);
     emit trackAdded(playlistId, trackId, position);
-    emit tracksChanged(QSet<int>{playlistId});
+    emit tracksAdded(QSet<int>{playlistId});
+    emit playlistContentChanged(QSet<int>{playlistId});
     return true;
 }
 
@@ -733,7 +740,7 @@ int PlaylistDAO::insertTracksIntoPlaylist(const QList<TrackId>& trackIds,
         return 0;
     }
 
-    int tracksAdded = 0;
+    int numTracksAdded = 0;
     ScopedTransaction transaction(m_database);
 
     int max_position = getMaxPosition(playlistId) + 1;
@@ -777,7 +784,7 @@ int PlaylistDAO::insertTracksIntoPlaylist(const QList<TrackId>& trackIds,
 
         // Increment the insert position for the track.
         ++insertPositon;
-        ++tracksAdded;
+        ++numTracksAdded;
     }
 
     transaction.commit();
@@ -788,8 +795,8 @@ int PlaylistDAO::insertTracksIntoPlaylist(const QList<TrackId>& trackIds,
         // TODO(XXX) The position is wrong if any track failed to insert.
         emit trackAdded(playlistId, trackId, insertPositon++);
     }
-    emit tracksChanged(QSet<int>{playlistId});
-    return tracksAdded;
+    emit tracksAdded(QSet<int>{playlistId});
+    return numTracksAdded;
 }
 
 void PlaylistDAO::addPlaylistToAutoDJQueue(const int playlistId, AutoDJSendLoc loc) {
@@ -912,7 +919,8 @@ bool PlaylistDAO::copyPlaylistTracks(const int sourcePlaylistID, const int targe
         m_playlistsTrackIsIn.insert(copiedTrackId, targetPlaylistID);
         emit trackAdded(targetPlaylistID, copiedTrackId, copiedPosition);
     }
-    emit tracksChanged(QSet<int>{targetPlaylistID});
+    emit tracksAdded(QSet<int>{targetPlaylistID});
+    emit playlistContentChanged(QSet<int>{targetPlaylistID});
     return true;
 }
 
@@ -936,7 +944,7 @@ int PlaylistDAO::getMaxPosition(const int playlistId) const {
     return position;
 }
 
-void PlaylistDAO::removeTracksFromPlaylists(const QList<TrackId>& trackIds) {
+void PlaylistDAO::removeTracksFromPlaylists(const QList<TrackId>& trackIds, bool purged) {
     // copy the hash, because there is no guarantee that "it" is valid after remove
     QMultiHash<TrackId, int> playlistsTrackIsInCopy = m_playlistsTrackIsIn;
     QSet<int> playlistIds;
@@ -959,7 +967,17 @@ void PlaylistDAO::removeTracksFromPlaylists(const QList<TrackId>& trackIds) {
     }
     transaction.commit();
 
-    emit tracksChanged(playlistIds);
+    // update the sidebar
+    emit playlistContentChanged(playlistIds);
+    // If this is called by TrackCollection::purgeTracks() it will call
+    // TrackDAO::afterPurgingTracks() afterwards which will enforce a model update
+    // (select()), so we don't need to signal PlaylistTableModel to select(),
+    // Also, this double select() can cause issues in BaseTrackCache.
+    // See https://github.com/mixxxdj/mixxx/issues/13111
+    if (purged) {
+        return;
+    }
+    emit tracksRemoved(playlistIds);
 }
 
 int PlaylistDAO::tracksInPlaylist(const int playlistId) const {
@@ -1045,7 +1063,7 @@ void PlaylistDAO::moveTrack(const int playlistId, const int oldPosition, const i
         qDebug() << query.lastError();
     }
 
-    emit tracksChanged(QSet<int>{playlistId});
+    emit tracksMoved(QSet<int>{playlistId});
 }
 
 void PlaylistDAO::searchForDuplicateTrack(const int fromPosition,
@@ -1251,7 +1269,7 @@ void PlaylistDAO::shuffleTracks(const int playlistId,
     }
 
     transaction.commit();
-    emit tracksChanged(QSet<int>{playlistId});
+    emit tracksMoved(QSet<int>{playlistId});
 }
 
 bool PlaylistDAO::isTrackInPlaylist(TrackId trackId, const int playlistId) const {
