@@ -1,44 +1,39 @@
-// library.cpp
-// Created 8/23/2009 by RJ Ryan (rryan@mit.edu)
-
 #include "library/library.h"
 
+#include <QApplication>
 #include <QDir>
-#include <QItemSelectionModel>
 #include <QMessageBox>
-#include <QPointer>
-#include <QTranslator>
 
+#include "control/controlobject.h"
 #include "controllers/keyboard/keyboardeventfilter.h"
-#include "database/mixxxdb.h"
 #include "library/analysisfeature.h"
 #include "library/autodj/autodjfeature.h"
 #include "library/banshee/bansheefeature.h"
 #include "library/browse/browsefeature.h"
-#include "library/crate/cratefeature.h"
+#ifdef __ENGINEPRIME__
+#include "library/export/libraryexporter.h"
+#endif
 #include "library/externaltrackcollection.h"
 #include "library/itunes/itunesfeature.h"
-#include "library/library_preferences.h"
+#include "library/library_prefs.h"
 #include "library/librarycontrol.h"
 #include "library/libraryfeature.h"
-#include "library/librarytablemodel.h"
 #include "library/mixxxlibraryfeature.h"
-#include "library/playlistfeature.h"
 #include "library/recording/recordingfeature.h"
 #include "library/rekordbox/rekordboxfeature.h"
 #include "library/rhythmbox/rhythmboxfeature.h"
 #include "library/serato/seratofeature.h"
-#include "library/setlogfeature.h"
 #include "library/sidebarmodel.h"
 #include "library/trackcollection.h"
 #include "library/trackcollectionmanager.h"
 #include "library/trackmodel.h"
+#include "library/trackset/crate/cratefeature.h"
+#include "library/trackset/playlistfeature.h"
+#include "library/trackset/setlogfeature.h"
 #include "library/traktor/traktorfeature.h"
 #include "mixer/playermanager.h"
 #include "moc_library.cpp"
-#include "recording/recordingmanager.h"
 #include "util/assert.h"
-#include "util/db/dbconnectionpooled.h"
 #include "util/logger.h"
 #include "util/sandbox.h"
 #include "widget/wlibrary.h"
@@ -50,10 +45,9 @@ namespace {
 
 const mixxx::Logger kLogger("Library");
 
-} // anonymous namespace
+} // namespace
 
-//static
-const QString Library::kConfigGroup("[Library]");
+using namespace mixxx::library::prefs;
 
 // This is the name which we use to register the WTrackTableView with the
 // WLibrary
@@ -69,20 +63,21 @@ Library::Library(
         TrackCollectionManager* pTrackCollectionManager,
         PlayerManager* pPlayerManager,
         RecordingManager* pRecordingManager)
-    : QObject(parent),
-      m_pConfig(pConfig),
-      m_pDbConnectionPool(std::move(pDbConnectionPool)),
-      m_pTrackCollectionManager(pTrackCollectionManager),
-      m_pSidebarModel(make_parented<SidebarModel>(this)),
-      m_pLibraryControl(make_parented<LibraryControl>(this)),
-      m_pMixxxLibraryFeature(nullptr),
-      m_pPlaylistFeature(nullptr),
-      m_pCrateFeature(nullptr),
-      m_pAnalysisFeature(nullptr) {
+        : QObject(parent),
+          m_pConfig(pConfig),
+          m_pDbConnectionPool(std::move(pDbConnectionPool)),
+          m_pTrackCollectionManager(pTrackCollectionManager),
+          m_pSidebarModel(make_parented<SidebarModel>(this)),
+          m_pLibraryControl(make_parented<LibraryControl>(this)),
+          m_pLibraryWidget(nullptr),
+          m_pMixxxLibraryFeature(nullptr),
+          m_pPlaylistFeature(nullptr),
+          m_pCrateFeature(nullptr),
+          m_pAnalysisFeature(nullptr) {
+    qRegisterMetaType<LibraryRemovalType>("LibraryRemovalType");
 
-    qRegisterMetaType<Library::RemovalType>("Library::RemovalType");
-
-    m_pKeyNotation.reset(new ControlObject(ConfigKey(kConfigGroup, "key_notation")));
+    m_pKeyNotation.reset(
+            new ControlObject(mixxx::library::prefs::kKeyNotationConfigKey));
 
     connect(m_pTrackCollectionManager,
             &TrackCollectionManager::libraryScanFinished,
@@ -95,12 +90,33 @@ Library::Library(
             this,
             m_pConfig);
     addFeature(m_pMixxxLibraryFeature);
+#ifdef __ENGINEPRIME__
+    connect(m_pMixxxLibraryFeature,
+            &MixxxLibraryFeature::exportLibrary,
+            this,
+            &Library::exportLibrary,
+            Qt::DirectConnection /* signal-to-signal */);
+#endif
 
     addFeature(new AutoDJFeature(this, m_pConfig, pPlayerManager));
+
     m_pPlaylistFeature = new PlaylistFeature(this, UserSettingsPointer(m_pConfig));
     addFeature(m_pPlaylistFeature);
+
     m_pCrateFeature = new CrateFeature(this, m_pConfig);
     addFeature(m_pCrateFeature);
+#ifdef __ENGINEPRIME__
+    connect(m_pCrateFeature,
+            &CrateFeature::exportAllCrates,
+            this,
+            &Library::exportLibrary, // signal-to-signal
+            Qt::DirectConnection);
+    connect(m_pCrateFeature,
+            &CrateFeature::exportCrate,
+            this,
+            &Library::exportCrate, // signal-to-signal
+            Qt::DirectConnection);
+#endif
 
     m_pBrowseFeature = new BrowseFeature(
             this, m_pConfig, pRecordingManager);
@@ -119,40 +135,57 @@ Library::Library(
     addFeature(m_pBrowseFeature);
 
     addFeature(new RecordingFeature(this, m_pConfig, pRecordingManager));
+
     addFeature(new SetlogFeature(this, UserSettingsPointer(m_pConfig)));
 
     m_pAnalysisFeature = new AnalysisFeature(this, m_pConfig);
-    connect(m_pPlaylistFeature, &PlaylistFeature::analyzeTracks,
-            m_pAnalysisFeature, &AnalysisFeature::analyzeTracks);
-    connect(m_pCrateFeature, &CrateFeature::analyzeTracks,
-            m_pAnalysisFeature, &AnalysisFeature::analyzeTracks);
+    connect(m_pPlaylistFeature,
+            &PlaylistFeature::analyzeTracks,
+            m_pAnalysisFeature,
+            &AnalysisFeature::analyzeTracks);
+    connect(m_pCrateFeature,
+            &CrateFeature::analyzeTracks,
+            m_pAnalysisFeature,
+            &AnalysisFeature::analyzeTracks);
+    connect(this,
+            &Library::analyzeTracks,
+            m_pAnalysisFeature,
+            &AnalysisFeature::analyzeTracks);
     addFeature(m_pAnalysisFeature);
     // Suspend a batch analysis while an ad-hoc analysis of
     // loaded tracks is in progress and resume it afterwards.
-    connect(pPlayerManager, &PlayerManager::trackAnalyzerProgress,
-            this, &Library::onPlayerManagerTrackAnalyzerProgress);
-    connect(pPlayerManager, &PlayerManager::trackAnalyzerIdle,
-            this, &Library::onPlayerManagerTrackAnalyzerIdle);
+    connect(pPlayerManager,
+            &PlayerManager::trackAnalyzerProgress,
+            this,
+            &Library::onPlayerManagerTrackAnalyzerProgress);
+    connect(pPlayerManager,
+            &PlayerManager::trackAnalyzerIdle,
+            this,
+            &Library::onPlayerManagerTrackAnalyzerIdle);
 
-    //iTunes and Rhythmbox should be last until we no longer have an obnoxious
-    //messagebox popup when you select them. (This forces you to reach for your
-    //mouse or keyboard if you're using MIDI control and you scroll through them...)
+    // iTunes and Rhythmbox should be last until we no longer have an obnoxious
+    // messagebox popup when you select them. (This forces you to reach for your
+    // mouse or keyboard if you're using MIDI control and you scroll through them...)
     if (RhythmboxFeature::isSupported() &&
-        m_pConfig->getValue(ConfigKey(kConfigGroup,"ShowRhythmboxLibrary"), true)) {
+            m_pConfig->getValue(
+                    ConfigKey(kConfigGroup, "ShowRhythmboxLibrary"), true)) {
         addFeature(new RhythmboxFeature(this, m_pConfig));
     }
-    if (m_pConfig->getValue(ConfigKey(kConfigGroup,"ShowBansheeLibrary"), true)) {
+    if (m_pConfig->getValue(
+                ConfigKey(kConfigGroup, "ShowBansheeLibrary"), true)) {
         BansheeFeature::prepareDbPath(m_pConfig);
         if (BansheeFeature::isSupported()) {
             addFeature(new BansheeFeature(this, m_pConfig));
         }
     }
     if (ITunesFeature::isSupported() &&
-        m_pConfig->getValue(ConfigKey(kConfigGroup,"ShowITunesLibrary"), true)) {
+            m_pConfig->getValue(
+                    ConfigKey(kConfigGroup, "ShowITunesLibrary"), true)) {
         addFeature(new ITunesFeature(this, m_pConfig));
     }
     if (TraktorFeature::isSupported() &&
-        m_pConfig->getValue(ConfigKey(kConfigGroup,"ShowTraktorLibrary"), true)) {
+            m_pConfig->getValue(
+                    ConfigKey(kConfigGroup, "ShowTraktorLibrary"), true)) {
         addFeature(new TraktorFeature(this, m_pConfig));
     }
 
@@ -160,41 +193,57 @@ Library::Library(
     // dynamically appear/disappear when correctly prepared removable devices
     // are mounted/unmounted would be to have some form of timed thread to check
     // periodically. Not ideal performance wise.
-    if (m_pConfig->getValue(ConfigKey(kConfigGroup, "ShowRekordboxLibrary"), true)) {
+    if (m_pConfig->getValue(
+                ConfigKey(kConfigGroup, "ShowRekordboxLibrary"), true)) {
         addFeature(new RekordboxFeature(this, m_pConfig));
     }
 
-    if (m_pConfig->getValue(ConfigKey(kConfigGroup, "ShowSeratoLibrary"), true)) {
+    if (m_pConfig->getValue(
+                ConfigKey(kConfigGroup, "ShowSeratoLibrary"), true)) {
         addFeature(new SeratoFeature(this, m_pConfig));
     }
 
     for (const auto& externalTrackCollection : m_pTrackCollectionManager->externalCollections()) {
         auto* feature = externalTrackCollection->newLibraryFeature(this, m_pConfig);
         if (feature) {
-            kLogger.info()
-                    << "Adding library feature for"
-                    << externalTrackCollection->name();
+            kLogger.info() << "Adding library feature for"
+                           << externalTrackCollection->name();
             addFeature(feature);
         } else {
-            kLogger.info()
-                    << "Library feature for"
-                    << externalTrackCollection->name()
-                    << "is not available";
+            kLogger.info() << "Library feature for"
+                           << externalTrackCollection->name()
+                           << "is not available";
         }
     }
 
     // On startup we need to check if all of the user's library folders are
     // accessible to us. If the user is using a database from <1.12.0 with
     // sandboxing then we will need them to give us permission.
-    qDebug() << "Checking for access to user's library directories:";
-    foreach (QString directoryPath, getDirs()) {
-        bool hasAccess = Sandbox::askForAccess(directoryPath);
-        qDebug() << "Checking for access to" << directoryPath << ":" << hasAccess;
+    const auto rootDirs = m_pTrackCollectionManager->internalCollection()->loadRootDirs();
+    for (mixxx::FileInfo dirInfo : rootDirs) {
+        if (!dirInfo.exists() || !dirInfo.isDir()) {
+            kLogger.warning()
+                    << "Skipping access check for missing or invalid directory"
+                    << dirInfo;
+            continue;
+        }
+        if (Sandbox::askForAccess(&dirInfo)) {
+            kLogger.info()
+                    << "Access to directory"
+                    << dirInfo
+                    << "from sandbox granted";
+        } else {
+            kLogger.warning()
+                    << "Access to directory"
+                    << dirInfo
+                    << "from sandbox denied";
+        }
     }
 
     m_iTrackTableRowHeight = m_pConfig->getValue(
             ConfigKey(kConfigGroup, "RowHeight"), kDefaultRowHeightPx);
-    QString fontStr = m_pConfig->getValueString(ConfigKey(kConfigGroup, "Font"));
+    QString fontStr =
+            m_pConfig->getValueString(ConfigKey(kConfigGroup, "Font"));
     if (!fontStr.isEmpty()) {
         m_trackTableFont.fromString(fontStr);
     } else {
@@ -202,19 +251,47 @@ Library::Library(
     }
 
     m_editMetadataSelectedClick = m_pConfig->getValue(
-            ConfigKey(kConfigGroup, "EditMetadataSelectedClick"),
-            PREF_LIBRARY_EDIT_METADATA_DEFAULT);
+            kEditMetadataSelectedClickConfigKey,
+            kEditMetadataSelectedClickDefault);
 }
 
-Library::~Library() {
-    // Empty but required due to forward declarations in header file!
-}
+Library::~Library() = default;
 
-TrackCollectionManager* Library::trackCollections() const {
+TrackCollectionManager* Library::trackCollectionManager() const {
     // Cannot be implemented inline due to forward declarations
     return m_pTrackCollectionManager;
 }
 
+namespace {
+class TrackAnalysisSchedulerEnvironmentImpl final : public TrackAnalysisSchedulerEnvironment {
+  public:
+    explicit TrackAnalysisSchedulerEnvironmentImpl(const Library* pLibrary)
+            : m_pLibrary(pLibrary) {
+        DEBUG_ASSERT(m_pLibrary);
+    }
+    ~TrackAnalysisSchedulerEnvironmentImpl() final = default;
+
+    TrackPointer loadTrackById(TrackId trackId) const final {
+        return m_pLibrary->trackCollectionManager()->getTrackById(trackId);
+    }
+
+  private:
+    // TODO: Use std::shared_ptr or std::weak_ptr instead of a plain pointer?
+    const Library* const m_pLibrary;
+};
+} // namespace
+
+TrackAnalysisScheduler::Pointer Library::createTrackAnalysisScheduler(
+        int numWorkerThreads,
+        AnalyzerModeFlags modeFlags) const {
+    return TrackAnalysisScheduler::createInstance(
+            std::make_unique<const TrackAnalysisSchedulerEnvironmentImpl>(this),
+            numWorkerThreads,
+            m_pDbConnectionPool,
+            m_pConfig,
+            modeFlags);
+}
+
 void Library::stopPendingTasks() {
     if (m_pAnalysisFeature) {
         m_pAnalysisFeature->stopAnalysis();
@@ -241,6 +318,10 @@ void Library::bindSearchboxWidget(WSearchLineEdit* pSearchboxWidget) {
             &WSearchLineEdit::slotSetFont);
     emit setTrackTableFont(m_trackTableFont);
     m_pLibraryControl->bindSearchboxWidget(pSearchboxWidget);
+    connect(pSearchboxWidget,
+            &WSearchLineEdit::setLibraryFocus,
+            m_pLibraryControl,
+            &LibraryControl::setLibraryFocus);
 }
 
 void Library::bindSidebarWidget(WLibrarySidebar* pSidebarWidget) {
@@ -270,6 +351,19 @@ void Library::bindSidebarWidget(WLibrarySidebar* pSidebarWidget) {
             &WLibrarySidebar::rightClicked,
             m_pSidebarModel,
             &SidebarModel::rightClicked);
+    connect(pSidebarWidget,
+            &WLibrarySidebar::renameItem,
+            m_pSidebarModel,
+            &SidebarModel::renameItem);
+    connect(pSidebarWidget,
+            &WLibrarySidebar::deleteItem,
+            m_pSidebarModel,
+            &SidebarModel::deleteItem);
+
+    connect(pSidebarWidget,
+            &WLibrarySidebar::setLibraryFocus,
+            m_pLibraryControl,
+            &LibraryControl::setLibraryFocus);
 
     pSidebarWidget->slotSetFont(m_trackTableFont);
     connect(this,
@@ -277,20 +371,19 @@ void Library::bindSidebarWidget(WLibrarySidebar* pSidebarWidget) {
             pSidebarWidget,
             &WLibrarySidebar::slotSetFont);
 
-    for (const auto& feature : qAsConst(m_features)) {
+    for (const auto& feature : std::as_const(m_features)) {
         feature->bindSidebarWidget(pSidebarWidget);
     }
 }
 
-void Library::bindLibraryWidget(WLibrary* pLibraryWidget,
-                         KeyboardEventFilter* pKeyboard) {
-    WTrackTableView* pTrackTableView =
-            new WTrackTableView(
-                    pLibraryWidget,
-                    m_pConfig,
-                    m_pTrackCollectionManager,
-                    pLibraryWidget->getTrackTableBackgroundColorOpacity(),
-                    true);
+void Library::bindLibraryWidget(
+        WLibrary* pLibraryWidget, KeyboardEventFilter* pKeyboard) {
+    m_pLibraryWidget = pLibraryWidget;
+    WTrackTableView* pTrackTableView = new WTrackTableView(m_pLibraryWidget,
+            m_pConfig,
+            this,
+            m_pLibraryWidget->getTrackTableBackgroundColorOpacity(),
+            true);
     pTrackTableView->installEventFilter(pKeyboard);
     connect(this,
             &Library::showTrackModel,
@@ -304,13 +397,24 @@ void Library::bindLibraryWidget(WLibrary* pLibraryWidget,
             &WTrackTableView::loadTrackToPlayer,
             this,
             &Library::slotLoadTrackToPlayer);
-    pLibraryWidget->registerView(m_sTrackViewName, pTrackTableView);
+    m_pLibraryWidget->registerView(m_sTrackViewName, pTrackTableView);
 
     connect(this,
             &Library::switchToView,
-            pLibraryWidget,
+            m_pLibraryWidget,
             &WLibrary::switchToView);
-
+    connect(this,
+            &Library::saveModelState,
+            pTrackTableView,
+            &WTrackTableView::slotSaveCurrentViewState);
+    connect(this,
+            &Library::restoreModelState,
+            pTrackTableView,
+            &WTrackTableView::slotRestoreCurrentViewState);
+    connect(this,
+            &Library::selectTrack,
+            m_pLibraryWidget,
+            &WLibrary::slotSelectTrackInActiveTrackView);
     connect(pTrackTableView,
             &WTrackTableView::trackSelected,
             this,
@@ -329,10 +433,19 @@ void Library::bindLibraryWidget(WLibrary* pLibraryWidget,
             pTrackTableView,
             &WTrackTableView::setSelectedClick);
 
-    m_pLibraryControl->bindLibraryWidget(pLibraryWidget, pKeyboard);
+    m_pLibraryControl->bindLibraryWidget(m_pLibraryWidget, pKeyboard);
 
-    for (const auto& feature : qAsConst(m_features)) {
-        feature->bindLibraryWidget(pLibraryWidget, pKeyboard);
+    connect(m_pLibraryControl,
+            &LibraryControl::showHideTrackMenu,
+            pTrackTableView,
+            &WTrackTableView::slotShowHideTrackMenu);
+    connect(pTrackTableView,
+            &WTrackTableView::trackMenuVisible,
+            m_pLibraryControl,
+            &LibraryControl::slotUpdateTrackMenuControl);
+
+    for (const auto& feature : std::as_const(m_features)) {
+        feature->bindLibraryWidget(m_pLibraryWidget, pKeyboard);
     }
 
     // Set the current font and row height on all the WTrackTableViews that were
@@ -380,10 +493,18 @@ void Library::addFeature(LibraryFeature* feature) {
             &LibraryFeature::trackSelected,
             this,
             &Library::trackSelected);
+    connect(feature,
+            &LibraryFeature::saveModelState,
+            this,
+            &Library::saveModelState);
+    connect(feature,
+            &LibraryFeature::restoreModelState,
+            this,
+            &Library::restoreModelState);
 }
 
 void Library::onPlayerManagerTrackAnalyzerProgress(
-        TrackId /*trackId*/,AnalyzerProgress /*analyzerProgress*/) {
+        TrackId /*trackId*/, AnalyzerProgress /*analyzerProgress*/) {
     if (m_pAnalysisFeature) {
         m_pAnalysisFeature->suspendAnalysis();
     }
@@ -415,21 +536,22 @@ void Library::slotLoadTrack(TrackPointer pTrack) {
     emit loadTrack(pTrack);
 }
 
-void Library::slotLoadLocationToPlayer(const QString& location, const QString& group) {
-    auto trackRef = TrackRef::fromFileInfo(location);
+void Library::slotLoadLocationToPlayer(const QString& location, const QString& group, bool play) {
+    auto trackRef = TrackRef::fromFilePath(location);
     TrackPointer pTrack = m_pTrackCollectionManager->getOrAddTrack(trackRef);
     if (pTrack) {
-        emit loadTrackToPlayer(pTrack, group);
+        emit loadTrackToPlayer(pTrack, group, play);
     }
 }
 
-void Library::slotLoadTrackToPlayer(TrackPointer pTrack, const QString& group, bool play) {
+void Library::slotLoadTrackToPlayer(
+        TrackPointer pTrack, const QString& group, bool play) {
     emit loadTrackToPlayer(pTrack, group, play);
 }
 
 void Library::slotRefreshLibraryModels() {
-   m_pMixxxLibraryFeature->refreshLibraryModels();
-   m_pAnalysisFeature->refreshLibraryModels();
+    m_pMixxxLibraryFeature->refreshLibraryModels();
+    m_pAnalysisFeature->refreshLibraryModels();
 }
 
 void Library::slotCreatePlaylist() {
@@ -453,9 +575,9 @@ void Library::slotRequestAddDir(const QString& dir) {
     // to canonicalize the path so we first wrap the directory string with a
     // QDir.
     QDir directory(dir);
-    Sandbox::createSecurityToken(directory);
+    Sandbox::createSecurityTokenForDir(directory);
 
-    if (!m_pTrackCollectionManager->addDirectory(dir)) {
+    if (!m_pTrackCollectionManager->addDirectory(mixxx::FileInfo(dir))) {
         QMessageBox::information(nullptr,
                 tr("Add Directory to Library"),
                 tr("Could not add the directory to your library. Either this "
@@ -464,21 +586,26 @@ void Library::slotRequestAddDir(const QString& dir) {
     }
     // set at least one directory in the config file so that it will be possible
     // to downgrade from 1.12
-    if (m_pConfig->getValueString(PREF_LEGACY_LIBRARY_DIR).length() < 1) {
-        m_pConfig->set(PREF_LEGACY_LIBRARY_DIR, dir);
+    if (m_pConfig->getValueString(kLegacyDirectoryConfigKey).length() < 1) {
+        m_pConfig->set(kLegacyDirectoryConfigKey, dir);
     }
 }
 
-void Library::slotRequestRemoveDir(const QString& dir, RemovalType removalType) {
+void Library::slotRequestRemoveDir(const QString& dir, LibraryRemovalType removalType) {
+    // Remove the directory from the directory list.
+    if (!m_pTrackCollectionManager->removeDirectory(mixxx::FileInfo(dir))) {
+        return;
+    }
+
     switch (removalType) {
-    case RemovalType::KeepTracks:
+    case LibraryRemovalType::KeepTracks:
         break;
-    case RemovalType::HideTracks:
+    case LibraryRemovalType::HideTracks:
         // Mark all tracks in this directory as deleted but DON'T purge them
         // in case the user re-adds them manually.
         m_pTrackCollectionManager->hideAllTracks(dir);
         break;
-    case RemovalType::PurgeTracks:
+    case LibraryRemovalType::PurgeTracks:
         // The user requested that we purge all metadata.
         m_pTrackCollectionManager->purgeAllTracks(dir);
         break;
@@ -486,21 +613,19 @@ void Library::slotRequestRemoveDir(const QString& dir, RemovalType removalType)
         DEBUG_ASSERT(!"unreachable");
     }
 
-    // Remove the directory from the directory list.
-    m_pTrackCollectionManager->removeDirectory(dir);
-
     // Also update the config file if necessary so that downgrading is still
     // possible.
-    QString confDir = m_pConfig->getValueString(PREF_LEGACY_LIBRARY_DIR);
+    QString confDir = m_pConfig->getValueString(kLegacyDirectoryConfigKey);
 
     if (QDir(dir) == QDir(confDir)) {
-        QStringList dirList = getDirs();
-        if (!dirList.isEmpty()) {
-            m_pConfig->set(PREF_LEGACY_LIBRARY_DIR, dirList.first());
-        } else {
+        const QList<mixxx::FileInfo> dirList =
+                m_pTrackCollectionManager->internalCollection()->loadRootDirs();
+        if (dirList.isEmpty()) {
             // Save empty string so that an old version of mixxx knows it has to
             // ask for a new directory.
-            m_pConfig->set(PREF_LEGACY_LIBRARY_DIR, QString());
+            m_pConfig->set(kLegacyDirectoryConfigKey, QString());
+        } else {
+            m_pConfig->set(kLegacyDirectoryConfigKey, dirList.first().location());
         }
     }
 }
@@ -510,16 +635,12 @@ void Library::slotRequestRelocateDir(const QString& oldDir, const QString& newDi
 
     // also update the config file if necessary so that downgrading is still
     // possible
-    QString conDir = m_pConfig->getValueString(PREF_LEGACY_LIBRARY_DIR);
+    QString conDir = m_pConfig->getValueString(kLegacyDirectoryConfigKey);
     if (oldDir == conDir) {
-        m_pConfig->set(PREF_LEGACY_LIBRARY_DIR, newDir);
+        m_pConfig->set(kLegacyDirectoryConfigKey, newDir);
     }
 }
 
-QStringList Library::getDirs() {
-    return m_pTrackCollectionManager->internalCollection()->getDirectoryDAO().getDirs();
-}
-
 void Library::setFont(const QFont& font) {
     QFontMetrics currMetrics(m_trackTableFont);
     QFontMetrics newMetrics(font);
@@ -545,8 +666,50 @@ void Library::setEditMedatataSelectedClick(bool enabled) {
     emit setSelectedClick(enabled);
 }
 
-TrackCollection& Library::trackCollection() {
-    DEBUG_ASSERT(m_pTrackCollectionManager);
-    DEBUG_ASSERT(m_pTrackCollectionManager->internalCollection());
-    return *m_pTrackCollectionManager->internalCollection();
+void Library::searchTracksInCollection(const QString& query) {
+    VERIFY_OR_DEBUG_ASSERT(m_pMixxxLibraryFeature) {
+        return;
+    }
+    m_pMixxxLibraryFeature->searchAndActivate(query);
+    emit switchToView(m_sTrackViewName);
+    m_pSidebarModel->activateDefaultSelection();
+}
+
+#ifdef __ENGINEPRIME__
+std::unique_ptr<mixxx::LibraryExporter> Library::makeLibraryExporter(
+        QWidget* parent) {
+    return std::make_unique<mixxx::LibraryExporter>(
+            parent, m_pConfig, m_pTrackCollectionManager);
+}
+#endif
+
+bool Library::isTrackIdInCurrentLibraryView(const TrackId& trackId) {
+    VERIFY_OR_DEBUG_ASSERT(trackId.isValid()) {
+        return false;
+    }
+    if (m_pLibraryWidget) {
+        return m_pLibraryWidget->isTrackInCurrentView(trackId);
+    } else {
+        return false;
+    }
+}
+
+void Library::slotSaveCurrentViewState() const {
+    if (m_pLibraryWidget) {
+        return m_pLibraryWidget->saveCurrentViewState();
+    }
+}
+
+void Library::slotRestoreCurrentViewState() const {
+    if (m_pLibraryWidget) {
+        return m_pLibraryWidget->restoreCurrentViewState();
+    }
+}
+
+LibraryTableModel* Library::trackTableModel() const {
+    VERIFY_OR_DEBUG_ASSERT(m_pMixxxLibraryFeature) {
+        return nullptr;
+    }
+
+    return m_pMixxxLibraryFeature->trackTableModel();
 }
