@@ -1,21 +1,26 @@
 #include "library/librarycontrol.h"
 
 #include <QApplication>
-#include <QItemSelectionModel>
+#include <QKeyEvent>
 #include <QModelIndex>
-#include <QModelIndexList>
+#include <QWindow>
 #include <QtDebug>
 
+#include "control/controlencoder.h"
 #include "control/controlobject.h"
 #include "control/controlpushbutton.h"
 #include "library/library.h"
 #include "library/libraryview.h"
 #include "mixer/playermanager.h"
 #include "moc_librarycontrol.cpp"
+#include "util/cmdlineargs.h"
 #include "widget/wlibrary.h"
 #include "widget/wlibrarysidebar.h"
 #include "widget/wsearchlineedit.h"
-#include "widget/wtracktableview.h"
+
+namespace {
+const QString kAppGroup = QStringLiteral("[App]");
+} // namespace
 
 LoadToGroupController::LoadToGroupController(LibraryControl* pParent, const QString& group)
         : QObject(pParent),
@@ -55,12 +60,15 @@ void LoadToGroupController::slotLoadToGroupAndPlay(double v) {
 LibraryControl::LibraryControl(Library* pLibrary)
         : QObject(pLibrary),
           m_pLibrary(pLibrary),
+          m_focusedWidget(FocusWidget::None),
+          m_prevFocusedWidget(FocusWidget::None),
           m_pLibraryWidget(nullptr),
           m_pSidebarWidget(nullptr),
           m_pSearchbox(nullptr),
-          m_numDecks("[Master]", "num_decks", this),
-          m_numSamplers("[Master]", "num_samplers", this),
-          m_numPreviewDecks("[Master]", "num_preview_decks", this) {
+          m_numDecks(kAppGroup, QStringLiteral("num_decks"), this),
+          m_numSamplers(kAppGroup, QStringLiteral("num_samplers"), this),
+          m_numPreviewDecks(kAppGroup, QStringLiteral("num_preview_decks"), this) {
+    qRegisterMetaType<FocusWidget>("FocusWidget");
 
     slotNumDecksChanged(m_numDecks.get());
     slotNumSamplersChanged(m_numSamplers.get());
@@ -73,6 +81,7 @@ LibraryControl::LibraryControl(Library* pLibrary)
     m_pMoveUp = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "MoveUp"));
     m_pMoveDown = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "MoveDown"));
     m_pMoveVertical = std::make_unique<ControlEncoder>(ConfigKey("[Library]", "MoveVertical"), false);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pMoveUp.get(),
             &ControlPushButton::valueChanged,
             this,
@@ -85,11 +94,13 @@ LibraryControl::LibraryControl(Library* pLibrary)
             &ControlEncoder::valueChanged,
             this,
             &LibraryControl::slotMoveVertical);
+#endif
 
     // Controls to navigate vertically within currently focused widget (up/down buttons)
     m_pScrollUp = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "ScrollUp"));
     m_pScrollDown = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "ScrollDown"));
     m_pScrollVertical = std::make_unique<ControlEncoder>(ConfigKey("[Library]", "ScrollVertical"), false);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pScrollUp.get(),
             &ControlPushButton::valueChanged,
             this,
@@ -102,11 +113,13 @@ LibraryControl::LibraryControl(Library* pLibrary)
             &ControlEncoder::valueChanged,
             this,
             &LibraryControl::slotScrollVertical);
+#endif
 
     // Controls to navigate horizontally within currently selected item (left/right buttons)
     m_pMoveLeft = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "MoveLeft"));
     m_pMoveRight = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "MoveRight"));
     m_pMoveHorizontal = std::make_unique<ControlEncoder>(ConfigKey("[Library]", "MoveHorizontal"), false);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pMoveLeft.get(),
             &ControlPushButton::valueChanged,
             this,
@@ -119,11 +132,14 @@ LibraryControl::LibraryControl(Library* pLibrary)
             &ControlEncoder::valueChanged,
             this,
             &LibraryControl::slotMoveHorizontal);
+#endif
 
-    // Control to navigate between widgets (tab/shit+tab button)
+    // Controls to navigate between widgets
+    // Relative focus controls (emulate Tab/Shift+Tab button press)
     m_pMoveFocusForward = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "MoveFocusForward"));
     m_pMoveFocusBackward = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "MoveFocusBackward"));
     m_pMoveFocus = std::make_unique<ControlEncoder>(ConfigKey("[Library]", "MoveFocus"), false);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pMoveFocusForward.get(),
             &ControlPushButton::valueChanged,
             this,
@@ -136,39 +152,88 @@ LibraryControl::LibraryControl(Library* pLibrary)
             &ControlEncoder::valueChanged,
             this,
             &LibraryControl::slotMoveFocus);
+#endif
+
+    // Direct focus control, read/write
+    m_pFocusedWidgetCO = std::make_unique<ControlPushButton>(
+            ConfigKey("[Library]", "focused_widget"));
+    m_pFocusedWidgetCO->setStates(static_cast<int>(FocusWidget::Count));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    m_pFocusedWidgetCO->connectValueChangeRequest(
+            this,
+            [this](double value) {
+                // Focus can not be removed from a widget just moved to another one.
+                // Thus, to keep the CO and QApplication::focusWidget() in sync we
+                // have to prevent scripts or GUI buttons setting the CO to 'None'.
+                // It's only set to 'None' internally when one of the library widgets
+                // receives a FocusOutEvent(), e.g. when the focus is moved to another
+                // widget, or when the main window loses focus.
+                const int valueInt = static_cast<int>(value);
+                if (valueInt != static_cast<int>(FocusWidget::None) &&
+                        valueInt < static_cast<int>(FocusWidget::Count)) {
+                    setLibraryFocus(static_cast<FocusWidget>(valueInt));
+                }
+            });
+#endif
+
+    // Pure trigger control. Alternative for signal/slot since widgets that want
+    // to call refocusPrevLibraryWidget() are cumbersome to connect to.
+    // This CO is never actually set or read so the value just needs to be not 0
+    m_pRefocusPrevWidgetCO = std::make_unique<ControlPushButton>(
+            ConfigKey("[Library]", "refocus_prev_widget"));
+    m_pRefocusPrevWidgetCO->setButtonMode(ControlPushButton::TRIGGER);
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    m_pRefocusPrevWidgetCO->connectValueChangeRequest(this,
+            &LibraryControl::refocusPrevLibraryWidget);
+#endif
 
     // Control to "goto" the currently selected item in focused widget (context dependent)
     m_pGoToItem = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "GoToItem"));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pGoToItem.get(),
             &ControlPushButton::valueChanged,
             this,
             &LibraryControl::slotGoToItem);
+#endif
 
     // Auto DJ controls
     m_pAutoDjAddTop = std::make_unique<ControlPushButton>(ConfigKey("[Library]","AutoDjAddTop"));
+    m_pAutoDjAddTop->addAlias(ConfigKey(
+            QStringLiteral("[Playlist]"), QStringLiteral("AutoDjAddTop")));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pAutoDjAddTop.get(),
             &ControlPushButton::valueChanged,
             this,
             &LibraryControl::slotAutoDjAddTop);
+#endif
 
     m_pAutoDjAddBottom = std::make_unique<ControlPushButton>(ConfigKey("[Library]","AutoDjAddBottom"));
+    m_pAutoDjAddBottom->addAlias(ConfigKey(
+            QStringLiteral("[Playlist]"), QStringLiteral("AutoDjAddBottom")));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pAutoDjAddBottom.get(),
             &ControlPushButton::valueChanged,
             this,
             &LibraryControl::slotAutoDjAddBottom);
+#endif
 
     m_pAutoDjAddReplace = std::make_unique<ControlPushButton>(
             ConfigKey("[Library]", "AutoDjAddReplace"));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pAutoDjAddReplace.get(),
             &ControlPushButton::valueChanged,
             this,
             &LibraryControl::slotAutoDjAddReplace);
+#endif
 
     // Sort controls
     m_pSortColumn = std::make_unique<ControlEncoder>(ConfigKey("[Library]", "sort_column"));
     m_pSortOrder = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "sort_order"));
     m_pSortOrder->setButtonMode(ControlPushButton::TOGGLE);
     m_pSortColumnToggle = std::make_unique<ControlEncoder>(ConfigKey("[Library]", "sort_column_toggle"), false);
+    m_pSortFocusedColumn = std::make_unique<ControlPushButton>(
+            ConfigKey("[Library]", "sort_focused_column"));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     connect(m_pSortColumn.get(),
             &ControlEncoder::valueChanged,
             this,
@@ -177,6 +242,14 @@ LibraryControl::LibraryControl(Library* pLibrary)
             &ControlEncoder::valueChanged,
             this,
             &LibraryControl::slotSortColumnToggle);
+    connect(m_pSortFocusedColumn.get(),
+            &ControlObject::valueChanged,
+            this,
+            [this](double value) {
+                if (value > 0.0) {
+                    slotSortColumnToggle(static_cast<int>(TrackModel::SortColumnId::CurrentIndex));
+                }
+            });
 
     // Font sizes
     m_pFontSizeKnob = std::make_unique<ControlObject>(
@@ -199,6 +272,7 @@ LibraryControl::LibraryControl(Library* pLibrary)
             &ControlPushButton::valueChanged,
             this,
             &LibraryControl::slotIncrementFontSize);
+#endif
 
     // Track Color controls
     m_pTrackColorPrev = std::make_unique<ControlPushButton>(ConfigKey("[Library]", "track_color_prev"));
@@ -212,7 +286,91 @@ LibraryControl::LibraryControl(Library* pLibrary)
             this,
             &LibraryControl::slotTrackColorNext);
 
-    /// Deprecated controls
+    // Controls to select saved searchbox queries and to clear the searchbox
+    m_pSelectHistoryNext = std::make_unique<ControlPushButton>(
+            ConfigKey("[Library]", "search_history_next"));
+    connect(m_pSelectHistoryNext.get(),
+            &ControlPushButton::valueChanged,
+            this,
+            [this](double value) {
+                VERIFY_OR_DEBUG_ASSERT(m_pSearchbox) {
+                    return;
+                }
+                if (value > 0.0) {
+                    m_pSearchbox->slotMoveSelectedHistory(1);
+                }
+            });
+    m_pSelectHistoryPrev = std::make_unique<ControlPushButton>(
+            ConfigKey("[Library]", "search_history_prev"));
+    connect(m_pSelectHistoryPrev.get(),
+            &ControlPushButton::valueChanged,
+            this,
+            [this](double value) {
+                VERIFY_OR_DEBUG_ASSERT(m_pSearchbox) {
+                    return;
+                }
+                if (value > 0.0) {
+                    m_pSearchbox->slotMoveSelectedHistory(-1);
+                }
+            });
+    m_pSelectHistorySelect = std::make_unique<ControlEncoder>(
+            ConfigKey("[Library]", "search_history_selector"), false);
+    connect(m_pSelectHistorySelect.get(),
+            &ControlEncoder::valueChanged,
+            this,
+            [this](double steps) {
+                VERIFY_OR_DEBUG_ASSERT(m_pSearchbox) {
+                    return;
+                }
+                int iSteps = static_cast<int>(steps);
+                if (iSteps) {
+                    m_pSearchbox->slotMoveSelectedHistory(static_cast<int>(steps));
+                }
+            });
+    m_pClearSearch = std::make_unique<ControlPushButton>(
+            ConfigKey("[Library]", "clear_search"));
+    connect(m_pClearSearch.get(),
+            &ControlPushButton::valueChanged,
+            this,
+            [this](double value) {
+                VERIFY_OR_DEBUG_ASSERT(m_pSearchbox) {
+                    return;
+                }
+                if (value > 0.0) {
+                    m_pSearchbox->slotClearSearch();
+                }
+            });
+    m_pDeleteSearchQuery = std::make_unique<ControlPushButton>(
+            ConfigKey("[Library]", "delete_search_query"));
+    connect(m_pDeleteSearchQuery.get(),
+            &ControlPushButton::valueChanged,
+            this,
+            [this](double value) {
+                VERIFY_OR_DEBUG_ASSERT(m_pSearchbox) {
+                    return;
+                }
+                if (value > 0.0) {
+                    m_pSearchbox->slotDeleteCurrentItem();
+                }
+            });
+
+    // Show the track context menu for selected tracks, or hide it
+    // if it is the current active window
+    // The control is updated in slotUpdateTrackMenuControl with the actual state
+    // sent from WTrackMenu via WTrackTableView
+    m_pShowTrackMenu = std::make_unique<ControlPushButton>(
+            ConfigKey("[Library]", "show_track_menu"));
+    m_pShowTrackMenu->setStates(2);
+    m_pShowTrackMenu->connectValueChangeRequest(this,
+            [this](double value) {
+                VERIFY_OR_DEBUG_ASSERT(m_pLibraryWidget) {
+                    return;
+                }
+                bool show = static_cast<bool>(value);
+                emit showHideTrackMenu(show);
+            });
+
+    // Deprecated controls
     m_pSelectNextTrack = std::make_unique<ControlPushButton>(ConfigKey("[Playlist]", "SelectNextTrack"));
     connect(m_pSelectNextTrack.get(),
             &ControlPushButton::valueChanged,
@@ -263,8 +421,24 @@ LibraryControl::LibraryControl(Library* pLibrary)
             this,
             &LibraryControl::slotLoadSelectedIntoFirstStopped);
 
-    ControlDoublePrivate::insertAlias(ConfigKey("[Playlist]", "AutoDjAddTop"), ConfigKey("[Library]", "AutoDjAddTop"));
-    ControlDoublePrivate::insertAlias(ConfigKey("[Playlist]", "AutoDjAddBottom"), ConfigKey("[Library]", "AutoDjAddBottom"));
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    QApplication* app = qApp;
+    // Update controls if any widget in any Mixxx window gets or loses focus
+    connect(app,
+            &QApplication::focusChanged,
+            this,
+            &LibraryControl::slotFocusedWidgetChanged);
+    // Also update controls if the window focus changed.
+    // Even though any new menu window has focus and will receive keypress events
+    // it does NOT have a focused widget before the first click or keypress.
+    // Thus a QMenu popping up is not reported by focusChanged(oldWidget, newWidget).
+    // QApplication::focusWidget() is still that in the previously focused
+    // window (MixxxMainWindow for example).
+    connect(app,
+            &QGuiApplication::focusWindowChanged,
+            this,
+            &LibraryControl::updateFocusedWidgetControls);
+#endif
 }
 
 LibraryControl::~LibraryControl() = default;
@@ -299,7 +473,6 @@ void LibraryControl::slotNumSamplersChanged(double v) {
     }
 }
 
-
 void LibraryControl::slotNumPreviewDecksChanged(double v) {
     int iNumPreviewDecks = static_cast<int>(v);
 
@@ -340,18 +513,12 @@ void LibraryControl::bindSearchboxWidget(WSearchLineEdit* pSearchbox) {
         disconnect(m_pSearchbox, nullptr, this, nullptr);
     }
     m_pSearchbox = pSearchbox;
-    connect(this,
-            &LibraryControl::clearSearchIfClearButtonHasFocus,
-            m_pSearchbox,
-            &WSearchLineEdit::slotClearSearchIfClearButtonHasFocus);
     connect(m_pSearchbox,
             &WSearchLineEdit::destroyed,
             this,
             &LibraryControl::searchboxWidgetDeleted);
 }
 
-
-
 void LibraryControl::libraryWidgetDeleted() {
     m_pLibraryWidget = nullptr;
 }
@@ -364,6 +531,10 @@ void LibraryControl::searchboxWidgetDeleted() {
     m_pSearchbox = nullptr;
 }
 
+void LibraryControl::slotUpdateTrackMenuControl(bool visible) {
+    m_pShowTrackMenu->setAndConfirm(visible ? 1.0 : 0.0);
+}
+
 void LibraryControl::slotLoadSelectedTrackToGroup(const QString& group, bool play) {
     if (!m_pLibraryWidget) {
         return;
@@ -386,7 +557,7 @@ void LibraryControl::slotLoadSelectedIntoFirstStopped(double v) {
         if (!pActiveView) {
             return;
         }
-        pActiveView->loadSelectedTrack();
+        pActiveView->activateSelectedTrack();
     }
 }
 
@@ -447,42 +618,93 @@ void LibraryControl::slotSelectTrack(double v) {
         return;
     }
 
-    int i = (int)v;
-
     LibraryView* pActiveView = m_pLibraryWidget->getActiveView();
     if (!pActiveView) {
         return;
     }
+
+    int i = (int)v;
     pActiveView->moveSelection(i);
 }
 
 void LibraryControl::slotMoveUp(double v) {
     if (v > 0) {
-        emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_Up, Qt::NoModifier});
+        slotMoveVertical(-1);
     }
 }
 
 void LibraryControl::slotMoveDown(double v) {
     if (v > 0) {
-        emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_Down, Qt::NoModifier});
+        slotMoveVertical(1);
     }
 }
 
 void LibraryControl::slotMoveVertical(double v) {
-    const auto key = (v < 0) ? Qt::Key_Up: Qt::Key_Down;
+    if (v == 0) {
+        return;
+    }
+
+    switch (m_focusedWidget) {
+    case FocusWidget::Sidebar: {
+        int i = static_cast<int>(v);
+        slotSelectSidebarItem(i);
+        return;
+    }
+    case FocusWidget::TracksTable: {
+        // `WLibraryTableView`'s cursor movement function has been overridden to
+        // wrap the selection around at the top/bottom of the tracks list. This
+        // behavior is thus shared between `[Library],MoveVertical` and Up/Down
+        // cursor key presses. See `WLibraryTableView::moveCursor()` for an
+        // explanation on why this is useful.
+        break;
+    }
+    case FocusWidget::Dialog: {
+        // For navigating dialogs map up/down to Tab/BackTab
+        // Don't use Shift + Tab! (see moveFocus())
+        const auto key = (v > 0) ? Qt::Key_Tab : Qt::Key_Backtab;
+        const auto times = static_cast<unsigned short>(std::abs(v));
+        emitKeyEvent(QKeyEvent{
+                QEvent::KeyPress, key, Qt::NoModifier, QString(), false, times});
+        return;
+    }
+    case FocusWidget::ContextMenu: {
+        // To navigate menus (and activate menus that were just opened) send the
+        // keyEvent to focusWindow() (not focusWidget() like emitKeyEvent() does)
+        const auto key = (v < 0) ? Qt::Key_Up : Qt::Key_Down;
+        const auto times = static_cast<unsigned short>(std::abs(v));
+        QKeyEvent event = QKeyEvent{
+                QEvent::KeyPress, key, Qt::NoModifier, QString(), false, times};
+        QApplication::sendEvent(QApplication::focusWindow(), &event);
+        return;
+    }
+    case FocusWidget::Searchbar:
+        // There's also m_pSearchbox->slotMoveSelectedHistory but that wraps around
+        // at top/bottom. Doesn't match Up/Down key behaviour and may not be desired.
+        // Proceed and let emitkeyEvent deal with it.
+        break;
+    case FocusWidget::None:
+    case FocusWidget::Unknown:
+    default:
+        // 'Unknown' uncategorized widget like a QComboBox. Return to not alter
+        // any WBeatSpinBox or WEffectSelector
+        setLibraryFocus(FocusWidget::TracksTable);
+        return;
+    }
+    const auto key = (v < 0) ? Qt::Key_Up : Qt::Key_Down;
     const auto times = static_cast<unsigned short>(std::abs(v));
-    emitKeyEvent(QKeyEvent{QEvent::KeyPress, key, Qt::NoModifier, QString(), false, times});
+    emitKeyEvent(QKeyEvent{
+            QEvent::KeyPress, key, Qt::NoModifier, QString(), false, times});
 }
 
 void LibraryControl::slotScrollUp(double v) {
     if (v > 0) {
-        emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_PageUp, Qt::NoModifier});
+        slotScrollVertical(-1);
     }
 }
 
 void LibraryControl::slotScrollDown(double v) {
     if (v > 0) {
-        emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_PageDown, Qt::NoModifier});
+        slotScrollVertical(1);
     }
 }
 
@@ -494,13 +716,13 @@ void LibraryControl::slotScrollVertical(double v) {
 
 void LibraryControl::slotMoveLeft(double v) {
     if (v > 0) {
-        emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_Left, Qt::NoModifier});
+        slotMoveHorizontal(-1);
     }
 }
 
 void LibraryControl::slotMoveRight(double v) {
     if (v > 0) {
-        emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_Right, Qt::NoModifier});
+        slotMoveHorizontal(1);
     }
 }
 
@@ -512,48 +734,38 @@ void LibraryControl::slotMoveHorizontal(double v) {
 
 void LibraryControl::slotMoveFocusForward(double v) {
     if (v > 0) {
-        emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_Tab, Qt::NoModifier});
+        slotMoveFocus(1);
     }
 }
 
 void LibraryControl::slotMoveFocusBackward(double v) {
     if (v > 0) {
-        emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_Tab, Qt::ShiftModifier});
+        slotMoveFocus(-1);
     }
 }
 
 void LibraryControl::slotMoveFocus(double v) {
-    const auto shift = (v < 0) ? Qt::ShiftModifier: Qt::NoModifier;
+    // Don't use Key_Tab + ShiftModifier for moving focus backwards!
+    // This would indeed move the focus, though it has a significant side-effect
+    // compared to pressing Shift + Tab on a real keyboard:
+    // Shift would remain 'pressed' in the previously focused widget until it
+    // receives any keyEvent with Qt::NoModifier.
+    const auto key = (v > 0) ? Qt::Key_Tab : Qt::Key_Backtab;
     const auto times = static_cast<unsigned short>(std::abs(v));
-    emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_Tab, shift, QString(), false, times});
+    emitKeyEvent(QKeyEvent{
+            QEvent::KeyPress, key, Qt::NoModifier, QString(), false, times});
 }
 
 void LibraryControl::emitKeyEvent(QKeyEvent&& event) {
-    // Ensure there's a valid library widget that can receive keyboard focus.
-    // QApplication::focusWidget() is not sufficient here because it
-    // would return any focused widget like WOverview, WWaveform, QSpinBox
-    VERIFY_OR_DEBUG_ASSERT(m_pSidebarWidget) {
-        return;
-    }
-    VERIFY_OR_DEBUG_ASSERT(m_pLibraryWidget) {
-        return;
-    }
     if (!QApplication::focusWindow()) {
-        qDebug() << "Mixxx window is not focused, don't send key events";
+        qInfo() << "No Mixxx window, popup or menu has focus."
+                << "Don't send key events.";
         return;
     }
 
-    bool keyIsTab = event.key() == static_cast<int>(Qt::Key_Tab);
-
-    // If the main window has focus, any widget can receive Tab.
-    // Other keys should be sent to library widgets only to not
-    // accidentally alter spinboxes etc.
-    if (!keyIsTab && !m_pSidebarWidget->hasFocus()
-            && !m_pLibraryWidget->getActiveView()->hasFocus()) {
-        setLibraryFocus();
-    }
-    if (keyIsTab && !QApplication::focusWidget()){
-        setLibraryFocus();
+    if (m_focusedWidget == FocusWidget::None) {
+        setLibraryFocus(FocusWidget::TracksTable);
+        return;
     }
 
     // Send the event pointer to the currently focused widget
@@ -565,45 +777,135 @@ void LibraryControl::emitKeyEvent(QKeyEvent&& event) {
     }
 }
 
-void LibraryControl::setLibraryFocus() {
-    // TODO: Set the focus of the library panel directly instead of sending tab from sidebar
-    VERIFY_OR_DEBUG_ASSERT(m_pSidebarWidget) {
-        return;
+FocusWidget LibraryControl::getFocusedWidget() {
+    auto* focusWindow = QApplication::focusWindow();
+    if (!focusWindow) {
+        return FocusWidget::None;
     }
-    // Try to focus the sidebar.
-    m_pSidebarWidget->setFocus();
 
-    // This may have failed, for example when a Cover window still has focus,
-    // so make sure the sidebar is focused or we'll crash.
-    if (!m_pSidebarWidget->hasFocus()) {
+    // Any QMenu is focusWindow() but NOT focusWidget() before any menu item
+    // is highlighted, though it can already receive keypress events.
+    // Thus, test for focus window type first to catch open popups.
+    if (focusWindow->type() == Qt::Popup) {
+        // WMainMenuBar
+        // WTrackMenuClassWindow = WTrackMenu + submenus
+        // QMenuClassWindow      = e.g. sidebar context menu
+        // qt_edit_menuWindow    = QLineEdit/QCombobox context menu
+        // QComboBoxListView of WEffectSelector, WSearchLineEdit, ...
+        return FocusWidget::ContextMenu;
+    } else if (focusWindow->type() == Qt::Dialog) {
+        // DlgPreferencesDlgWindow
+        // DlgDeveloperToolsWindow
+        // DlgAboutDlgWindow
+        // DlgKeywheelWindow
+        // QInputDialogClassWindow (file dialogs, rename/create dialogs)
+        // error messages and Close Mixxx confirmation dialog
+        // ToDo(ronso0) handle CoverArt:
+        // - refocus tracks view?
+        // DlgCoverArtFullSizeWindow
+        return FocusWidget::Dialog;
+    }
+
+    // Now we assume MixxxMainWindow is focused
+    if (!QApplication::focusWidget()) {
+        return FocusWidget::None;
+    }
+
+    if (m_pSearchbox && m_pSearchbox->hasFocus()) {
+        return FocusWidget::Searchbar;
+    } else if (m_pSidebarWidget && m_pSidebarWidget->hasFocus()) {
+        return FocusWidget::Sidebar;
+    } else if (m_pLibraryWidget && m_pLibraryWidget->getActiveView()->hasFocus()) {
+        return FocusWidget::TracksTable;
+    } else {
+        // Unknown widget, for example Clear button in WSearcLineEdit,
+        // some drop-down view, WBeatSpinBox or QLineEdit in WtrackTableView
+        return FocusWidget::Unknown;
+    }
+}
+
+void LibraryControl::setLibraryFocus(FocusWidget newFocusWidget) {
+    if (!QApplication::focusWindow()) {
+        qInfo() << "No Mixxx window, popup or menu has focus."
+                << "Don't attempt to focus a specific widget.";
         return;
     }
-    // Send Tab to move focus to the Tracks table.
-    // Obviously only works as desired if the skin widgets are arranged
-    // accordingly.
-    QKeyEvent event(QEvent::KeyPress, Qt::Key_Tab, Qt::NoModifier);
-    QApplication::sendEvent(m_pSidebarWidget, &event);
+
+    // ignore no-op
+    if (newFocusWidget == m_focusedWidget) {
+        return;
+    }
+
+    switch (newFocusWidget) {
+    case FocusWidget::Searchbar:
+        VERIFY_OR_DEBUG_ASSERT(m_pSearchbox) {
+            return;
+        }
+        m_pSearchbox->setFocus();
+        return;
+    case FocusWidget::Sidebar:
+        VERIFY_OR_DEBUG_ASSERT(m_pSidebarWidget) {
+            return;
+        }
+        m_pSidebarWidget->setFocus();
+        return;
+    case FocusWidget::TracksTable:
+        VERIFY_OR_DEBUG_ASSERT(m_pLibraryWidget) {
+            return;
+        }
+        m_pLibraryWidget->getActiveView()->setFocus();
+        return;
+    case FocusWidget::None:
+        // What could be the goal, what are the consequences of manually
+        // removing focus from a widget?
+    default:
+        // Ignore invalid requests and don't allow focussing any other widget
+        // manually, like QDialog or QMenu.
+        return;
+    }
+    // Done. QApplication::focusChanged will invoke updateFocusControl()
+    // to update [Library],focused_widget
+}
+
+void LibraryControl::slotFocusedWidgetChanged(QWidget* oldW, QWidget* newW) {
+    Q_UNUSED(newW);
+
+    // If one of the library widgets had focus store it so we can return to it,
+    // for example when we finish editing a WBeatSizeSpinBox.
+    if (m_pSearchbox && oldW == m_pSearchbox) {
+        m_prevFocusedWidget = FocusWidget::Searchbar;
+    } else if (m_pSidebarWidget && oldW == m_pSidebarWidget) {
+        m_prevFocusedWidget = FocusWidget::Sidebar;
+    } else if (m_pLibraryWidget && oldW == m_pLibraryWidget->currentWidget()) {
+        m_prevFocusedWidget = FocusWidget::TracksTable;
+    }
+    updateFocusedWidgetControls();
+}
+
+void LibraryControl::updateFocusedWidgetControls() {
+    m_focusedWidget = getFocusedWidget();
+    // Update "[Library], focused_widget" control
+    double newVal = static_cast<double>(m_focusedWidget);
+    m_pFocusedWidgetCO->setAndConfirm(newVal);
+}
+
+void LibraryControl::refocusPrevLibraryWidget() {
+    setLibraryFocus(m_prevFocusedWidget);
 }
 
 void LibraryControl::slotSelectSidebarItem(double v) {
-    VERIFY_OR_DEBUG_ASSERT(m_pSidebarWidget) {
+    if (!m_pSidebarWidget) {
         return;
     }
-    if (v > 0) {
-        QApplication::postEvent(m_pSidebarWidget, new QKeyEvent(
-            QEvent::KeyPress,
-            (int)Qt::Key_Down, Qt::NoModifier, QString(), true));
-        QApplication::postEvent(m_pSidebarWidget, new QKeyEvent(
-            QEvent::KeyRelease,
-            (int)Qt::Key_Down, Qt::NoModifier, QString(), true));
-    } else if (v < 0) {
-        QApplication::postEvent(m_pSidebarWidget, new QKeyEvent(
-            QEvent::KeyPress,
-            (int)Qt::Key_Up, Qt::NoModifier, QString(), true));
-        QApplication::postEvent(m_pSidebarWidget, new QKeyEvent(
-            QEvent::KeyRelease,
-            (int)Qt::Key_Up, Qt::NoModifier, QString(), true));
+    if (v == 0) {
+        return;
     }
+
+    const auto key = (v < 0) ? Qt::Key_Up : Qt::Key_Down;
+    const auto times = static_cast<unsigned short>(std::abs(v));
+    QKeyEvent event = QKeyEvent{
+            QEvent::KeyPress, key, Qt::NoModifier, QString(), false, times};
+    QApplication::sendEvent(m_pSidebarWidget, &event);
 }
 
 void LibraryControl::slotSelectNextSidebarItem(double v) {
@@ -628,51 +930,50 @@ void LibraryControl::slotGoToItem(double v) {
     if (v <= 0) {
         return;
     }
-    VERIFY_OR_DEBUG_ASSERT(m_pSidebarWidget) {
-        return;
-    }
-    VERIFY_OR_DEBUG_ASSERT(m_pLibraryWidget) {
-        return;
-    }
-    VERIFY_OR_DEBUG_ASSERT(m_pSearchbox) {
-        return;
-    }
 
-    // Focus the library if this is a leaf node in the tree
-    if (m_pSidebarWidget->hasFocus()) {
+    switch (m_focusedWidget) {
+    case FocusWidget::Sidebar:
+        // Focus the library if this is a leaf node in the tree
         // Note that Tracks and AutoDJ always return 'false':
         // expanding those root items via controllers is considered dispensable
         // because the subfeatures' actions can't be accessed by controllers anyway.
         if (m_pSidebarWidget->isLeafNodeSelected()) {
-            setLibraryFocus();
-            return;
+            setLibraryFocus(FocusWidget::TracksTable);
         } else {
             // Otherwise toggle the sidebar item expanded state
-            slotToggleSelectedSidebarItem(v);
+            m_pSidebarWidget->toggleSelectedItem();
         }
-    }
-
-    // Load current track if a LibraryView object has focus
-    LibraryView* pActiveView = m_pLibraryWidget->getActiveView();
-    if (pActiveView && pActiveView->hasFocus()) {
-        pActiveView->loadSelectedTrack();
+        return;
+    case FocusWidget::TracksTable:
+        m_pLibraryWidget->getActiveView()->activateSelectedTrack();
+        return;
+    case FocusWidget::Dialog: {
+        // press & release Space (QAbstractButton::clicked() is emitted on release)
+        QKeyEvent pressSpace = QKeyEvent{QEvent::KeyPress, Qt::Key_Space, Qt::NoModifier};
+        QKeyEvent releaseSpace = QKeyEvent{QEvent::KeyRelease, Qt::Key_Space, Qt::NoModifier};
+        QApplication::sendEvent(QApplication::focusWindow(), &pressSpace);
+        QApplication::sendEvent(QApplication::focusWindow(), &releaseSpace);
         return;
     }
-
-    // Clear the search if the searchbox has focus
-    emit clearSearchIfClearButtonHasFocus();
-
-    // If the focused window is a dialog, press Enter
-    auto* focusWindow = QApplication::focusWindow();
-    if (focusWindow && (focusWindow->type() & (Qt::Dialog | Qt::Popup))) {
-        QKeyEvent event(QEvent::KeyPress, Qt::Key_Enter, Qt::NoModifier);
-        QApplication::sendEvent(focusWindow, &event);
+    case FocusWidget::ContextMenu:
+    case FocusWidget::Unknown: {
+        // press Return to
+        // * expand submenus or select highlighted menu item
+        // * click Clear button in WSearcLineEdit (Note: even though this QToolButton
+        //   inherits QAbstractButton, clicked is emitted on keypress if Return is used)
+        // * confirm and WBeatSpinBox and move focus to tracks table
+        // * ) and some more.
+        // If Unknown is some other 'untrained' or unresponsive widget
+        // GoToItem is inappropriate and we can't do much about that.
+        QKeyEvent event = QKeyEvent{QEvent::KeyPress, Qt::Key_Return, Qt::NoModifier};
+        QApplication::sendEvent(QApplication::focusWindow(), &event);
+        return;
+    }
+    case FocusWidget::Searchbar:
+    case FocusWidget::None:
+    default:
+        setLibraryFocus(FocusWidget::TracksTable);
     }
-
-    // TODO(xxx) instead of remote control the widgets individual, we should
-    // translate this into Alt+Return and handle it at each library widget
-    // individual https://bugs.launchpad.net/mixxx/+bug/1758618
-    //emitKeyEvent(QKeyEvent{QEvent::KeyPress, Qt::Key_Return, Qt::AltModifier});
 }
 
 void LibraryControl::slotSortColumn(double v) {
