@@ -16,12 +16,20 @@ const QString kModelName = "playlist:";
 PlaylistTableModel::PlaylistTableModel(QObject* parent,
         TrackCollectionManager* pTrackCollectionManager,
         const char* settingsNamespace,
-        bool keepDeletedTracks)
+        bool keepHiddenTracks)
         : TrackSetTableModel(parent, pTrackCollectionManager, settingsNamespace),
           m_iPlaylistId(kInvalidPlaylistId),
-          m_keepDeletedTracks(keepDeletedTracks) {
+          m_keepHiddenTracks(keepHiddenTracks) {
     connect(&m_pTrackCollectionManager->internalCollection()->getPlaylistDAO(),
-            &PlaylistDAO::tracksChanged,
+            &PlaylistDAO::tracksAdded,
+            this,
+            &PlaylistTableModel::playlistsChanged);
+    connect(&m_pTrackCollectionManager->internalCollection()->getPlaylistDAO(),
+            &PlaylistDAO::tracksMoved,
+            this,
+            &PlaylistTableModel::playlistsChanged);
+    connect(&m_pTrackCollectionManager->internalCollection()->getPlaylistDAO(),
+            &PlaylistDAO::tracksRemoved,
             this,
             &PlaylistTableModel::playlistsChanged);
 }
@@ -139,7 +147,7 @@ void PlaylistTableModel::selectPlaylist(int playlistId) {
 
     m_iPlaylistId = playlistId;
 
-    if (!m_keepDeletedTracks) {
+    if (!m_keepHiddenTracks) {
         // From Mixxx 2.1 we drop tracks that have been explicitly deleted
         // in the library (mixxx_deleted = 0) from playlists.
         // These invisible tracks, consuming a playlist position number were
@@ -359,6 +367,8 @@ TrackModel::Capabilities PlaylistTableModel::getCapabilities() const {
             Capability::LoadToSampler |
             Capability::LoadToPreviewDeck |
             Capability::ResetPlayed |
+            Capability::RemoveFromDisk |
+            Capability::Hide |
             Capability::Analyze;
 
     if (m_iPlaylistId !=
@@ -373,8 +383,9 @@ TrackModel::Capabilities PlaylistTableModel::getCapabilities() const {
     if (m_pTrackCollectionManager->internalCollection()
                     ->getPlaylistDAO()
                     .getHiddenType(m_iPlaylistId) == PlaylistDAO::PLHT_SET_LOG) {
-        // Disable track reordering and adding tracks via drag'n'drop for history playlists
-        caps &= ~(Capability::ReceiveDrops | Capability::Reorder);
+        // Disallow reordering and hiding tracks, as well as adding tracks via
+        // drag'n'drop for history playlists
+        caps &= ~(Capability::ReceiveDrops | Capability::Reorder | Capability::Hide);
     }
     bool locked = m_pTrackCollectionManager->internalCollection()->getPlaylistDAO().isPlaylistLocked(m_iPlaylistId);
     if (locked) {
