@@ -39,12 +39,16 @@ BasePlaylistFeature::BasePlaylistFeature(
         UserSettingsPointer pConfig,
         PlaylistTableModel* pModel,
         const QString& rootViewName,
-        const QString& iconName)
+        const QString& iconName,
+        const QString& countsDurationTableName,
+        bool keepHiddenTracks)
         : BaseTrackSetFeature(pLibrary, pConfig, rootViewName, iconName),
           m_playlistDao(pLibrary->trackCollectionManager()
                                 ->internalCollection()
                                 ->getPlaylistDAO()),
-          m_pPlaylistTableModel(pModel) {
+          m_pPlaylistTableModel(pModel),
+          m_countsDurationTableName(countsDurationTableName),
+          m_keepHiddenTracks(keepHiddenTracks) {
     pModel->setParent(this);
 
     initActions();
@@ -154,14 +158,14 @@ void BasePlaylistFeature::connectPlaylistDAO() {
             this,
             &BasePlaylistFeature::slotPlaylistTableChanged);
     connect(&m_playlistDao,
-            &PlaylistDAO::tracksChanged,
+            &PlaylistDAO::playlistContentChanged,
             this,
             &BasePlaylistFeature::slotPlaylistContentOrLockChanged);
     connect(&m_playlistDao,
             &PlaylistDAO::renamed,
             this,
-            // In "History") just the item is renamed, while in "Playlists" the
-            // entire sidebar model is rebuilt to resort items by name
+            // In "History" just the item is renamed, while in "Playlists" the
+            // entire sidebar model is rebuilt to re-sort items by name
             &BasePlaylistFeature::slotPlaylistTableRenamed);
 }
 
@@ -568,7 +572,8 @@ void BasePlaylistFeature::slotExportPlaylist() {
     QScopedPointer<PlaylistTableModel> pPlaylistTableModel(
             new PlaylistTableModel(this,
                     m_pLibrary->trackCollectionManager(),
-                    "mixxx.db.model.playlist_export"));
+                    "mixxx.db.model.playlist_export",
+                    m_keepHiddenTracks));
 
     emit saveModelState();
     pPlaylistTableModel->selectPlaylist(playlistId);
@@ -705,6 +710,46 @@ void BasePlaylistFeature::htmlLinkClicked(const QUrl& link) {
     }
 }
 
+QString BasePlaylistFeature::fetchPlaylistLabel(int playlistId) {
+    // This queries the temporary id/count/duration table that was has been created
+    // by the features' createPlaylistLabels() (updated each time playlists are added/removed)
+    QSqlDatabase database =
+            m_pLibrary->trackCollectionManager()->internalCollection()->database();
+    VERIFY_OR_DEBUG_ASSERT(database.tables(QSql::Views).contains(m_countsDurationTableName)) {
+        qWarning() << "BasePlaylistFeature: view" << m_countsDurationTableName
+                   << "does not exist! Can't fetch label for playlist" << playlistId;
+        return QString();
+    }
+    QSqlTableModel playlistTableModel(this, database);
+    playlistTableModel.setTable(m_countsDurationTableName);
+    const QString filter = "id=" + QString::number(playlistId);
+    playlistTableModel.setFilter(filter);
+    playlistTableModel.select();
+    while (playlistTableModel.canFetchMore()) {
+        playlistTableModel.fetchMore();
+    }
+    QSqlRecord record = playlistTableModel.record();
+    int nameColumn = record.indexOf("name");
+    int countColumn = record.indexOf("count");
+    int durationColumn = record.indexOf("durationSeconds");
+
+    DEBUG_ASSERT(playlistTableModel.rowCount() <= 1);
+    if (playlistTableModel.rowCount() > 0) {
+        QString name =
+                playlistTableModel.data(playlistTableModel.index(0, nameColumn))
+                        .toString();
+        int count = playlistTableModel
+                            .data(playlistTableModel.index(0, countColumn))
+                            .toInt();
+        int duration =
+                playlistTableModel
+                        .data(playlistTableModel.index(0, durationColumn))
+                        .toInt();
+        return createPlaylistLabel(name, count, duration);
+    }
+    return QString();
+}
+
 void BasePlaylistFeature::updateChildModel(const QSet<int>& playlistIds) {
     // qDebug() << "BasePlaylistFeature::updateChildModel() for"
     //          << playlistIds.count() << "playlist(s)";
@@ -738,6 +783,7 @@ void BasePlaylistFeature::updateChildModel(const QSet<int>& playlistIds) {
             }
         }
     }
+    m_pSidebarModel->triggerRepaint();
 }
 
 /// Clears the child model dynamically, but the invisible root item remains
@@ -802,6 +848,22 @@ void BasePlaylistFeature::markTreeItem(TreeItem* pTreeItem) {
     }
 }
 
+QString BasePlaylistFeature::createPlaylistLabel(const QString& name,
+        int count,
+        int duration) const {
+    // Show duration only if playlist has tracks
+    if (count > 0) {
+        return QStringLiteral("%1 (%2) %3")
+                .arg(name,
+                        QString::number(count),
+                        mixxx::Duration::formatTime(
+                                duration, mixxx::Duration::Precision::SECONDS));
+    } else {
+        return QStringLiteral("%1 (%2)").arg(name,
+                QString::number(count));
+    }
+}
+
 void BasePlaylistFeature::slotResetSelectedTrack() {
     slotTrackSelected(TrackId{});
 }
