@@ -33,9 +33,11 @@ SetlogFeature::SetlogFeature(
                           nullptr,
                           pLibrary->trackCollectionManager(),
                           "mixxx.db.model.setlog",
-                          /*keep deleted tracks*/ true),
+                          /*keep hidden tracks*/ true),
                   QStringLiteral("SETLOGHOME"),
-                  QStringLiteral("history")),
+                  QStringLiteral("history"),
+                  QStringLiteral("SetlogCountsDurations"),
+                  /*keep hidden tracks*/ true),
           m_currentPlaylistId(kInvalidPlaylistId),
           m_yearNodeId(kInvalidPlaylistId),
           m_pLibrary(pLibrary),
@@ -122,7 +124,7 @@ void SetlogFeature::deleteAllUnlockedPlaylistsWithFewerTracks() {
     int minTrackCount = m_pConfig->getValue(
             kHistoryMinTracksToKeepConfigKey,
             kHistoryMinTracksToKeepDefault);
-    m_playlistDao.deleteAllUnlockedPlaylistsWithFewerTracks(PlaylistDAO::HiddenType::PLHT_SET_LOG,
+    m_playlistDao.deleteAllUnlockedPlaylistsWithFewerTracks(PlaylistDAO::PLHT_SET_LOG,
             minTrackCount);
     transaction.commit();
 }
@@ -213,14 +215,42 @@ void SetlogFeature::onRightClickChild(const QPoint& globalPos, const QModelIndex
 /// Use a custom model in the history for grouping by year
 /// @param selectedId row which should be selected
 QModelIndex SetlogFeature::constructChildModel(int selectedId) {
-    // qDebug() << "SetlogFeature::constructChildModel() id:" << selectedId;
+    // qDebug() << "SetlogFeature::constructChildModel() selected:" << selectedId;
     // Setup the sidebar playlist model
-    QSqlTableModel playlistTableModel(this,
-            m_pLibrary->trackCollectionManager()->internalCollection()->database());
-    playlistTableModel.setTable("Playlists");
-    playlistTableModel.setFilter("hidden=" + QString::number(PlaylistDAO::PLHT_SET_LOG));
-    playlistTableModel.setSort(
-            playlistTableModel.fieldIndex("id"), Qt::DescendingOrder);
+    QSqlDatabase database =
+            m_pLibrary->trackCollectionManager()->internalCollection()->database();
+
+    QString queryString = QStringLiteral(
+            "CREATE TEMPORARY VIEW IF NOT EXISTS %1 "
+            "AS SELECT "
+            "  Playlists.id AS id, "
+            "  Playlists.name AS name, "
+            "  Playlists.date_created AS date_created, "
+            "  LOWER(Playlists.name) AS sort_name, "
+            "  max(PlaylistTracks.position) AS count,"
+            "  SUM(library.duration) AS durationSeconds "
+            "FROM Playlists "
+            "LEFT JOIN PlaylistTracks "
+            "  ON PlaylistTracks.playlist_id = Playlists.id "
+            "LEFT JOIN library "
+            "  ON PlaylistTracks.track_id = library.id "
+            "  WHERE Playlists.hidden = %2 "
+            "  GROUP BY Playlists.id")
+                                  .arg(m_countsDurationTableName,
+                                          QString::number(PlaylistDAO::PLHT_SET_LOG));
+    ;
+    queryString.append(
+            mixxx::DbConnection::collateLexicographically(
+                    " ORDER BY sort_name"));
+    QSqlQuery query(database);
+    if (!query.exec(queryString)) {
+        LOG_FAILED_QUERY(query);
+    }
+
+    // Setup the sidebar playlist model
+    QSqlTableModel playlistTableModel(this, database);
+    playlistTableModel.setTable(m_countsDurationTableName);
+    playlistTableModel.setSort(playlistTableModel.fieldIndex("id"), Qt::DescendingOrder);
     playlistTableModel.select();
     while (playlistTableModel.canFetchMore()) {
         playlistTableModel.fetchMore();
@@ -229,6 +259,8 @@ QModelIndex SetlogFeature::constructChildModel(int selectedId) {
     int nameColumn = record.indexOf("name");
     int idColumn = record.indexOf("id");
     int createdColumn = record.indexOf("date_created");
+    int countColumn = record.indexOf("count");
+    int durationColumn = record.indexOf("durationSeconds");
 
     // Nice to have: restore previous expanded/collapsed state of YEAR items
     clearChildModel();
@@ -250,8 +282,16 @@ QModelIndex SetlogFeature::constructChildModel(int selectedId) {
                 playlistTableModel
                         .data(playlistTableModel.index(row, createdColumn))
                         .toDateTime();
+        int count = playlistTableModel
+                            .data(playlistTableModel.index(row, countColumn))
+                            .toInt();
+        int duration =
+                playlistTableModel
+                        .data(playlistTableModel.index(row, durationColumn))
+                        .toInt();
+        QString label = createPlaylistLabel(name, count, duration);
 
-        // Create the TreeItem whose parent is the invisible root item
+        // Create the TreeItem whose parent is the invisible root item.
         // Show only [kNumToplevelHistoryEntries] recent playlists at the top level
         // before grouping them by year.
         if (row >= kNumToplevelHistoryEntries) {
@@ -273,12 +313,12 @@ QModelIndex SetlogFeature::constructChildModel(int selectedId) {
                 itemList.push_back(std::move(pNewGroupItem));
             }
 
-            TreeItem* pItem = pGroupItem->appendChild(name, id);
+            TreeItem* pItem = pGroupItem->appendChild(label, id);
             pItem->setBold(m_playlistIdsOfSelectedTrack.contains(id));
             decorateChild(pItem, id);
         } else {
             // add most recent top-level playlist
-            auto pItem = std::make_unique<TreeItem>(name, id);
+            auto pItem = std::make_unique<TreeItem>(label, id);
             pItem->setBold(m_playlistIdsOfSelectedTrack.contains(id));
             decorateChild(pItem.get(), id);
 
@@ -292,29 +332,6 @@ QModelIndex SetlogFeature::constructChildModel(int selectedId) {
     return indexFromPlaylistId(selectedId);
 }
 
-QString SetlogFeature::fetchPlaylistLabel(int playlistId) {
-    // Setup the sidebar playlist model
-    // TODO(ronso0) Why not m_playlistDao.getPlaylistName(id) ??
-    QSqlTableModel playlistTableModel(this,
-            m_pLibrary->trackCollectionManager()->internalCollection()->database());
-    playlistTableModel.setTable("Playlists");
-    QString filter = "id=" + QString::number(playlistId);
-    playlistTableModel.setFilter(filter);
-    playlistTableModel.select();
-    while (playlistTableModel.canFetchMore()) {
-        playlistTableModel.fetchMore();
-    }
-    QSqlRecord record = playlistTableModel.record();
-    int nameColumn = record.indexOf("name");
-
-    DEBUG_ASSERT(playlistTableModel.rowCount() <= 1);
-    if (playlistTableModel.rowCount() > 0) {
-        return playlistTableModel.data(playlistTableModel.index(0, nameColumn))
-                .toString();
-    }
-    return QString();
-}
-
 void SetlogFeature::decorateChild(TreeItem* item, int playlistId) {
     if (playlistId == m_currentPlaylistId) {
         item->setIcon(QIcon(":/images/library/ic_library_history_current.svg"));
@@ -686,7 +703,7 @@ void SetlogFeature::slotPlaylistTableChanged(int playlistId) {
 }
 
 void SetlogFeature::slotPlaylistContentOrLockChanged(const QSet<int>& playlistIds) {
-    // qDebug() << "slotPlaylistContentOrLockChanged() for"
+    // qDebug() << "SetlogFeature::slotPlaylistContentOrLockChanged() for"
     //          << playlistIds.count() << "playlist(s)";
     QSet<int> idsToBeUpdated;
     for (const auto playlistId : std::as_const(playlistIds)) {
@@ -699,7 +716,7 @@ void SetlogFeature::slotPlaylistContentOrLockChanged(const QSet<int>& playlistId
 
 void SetlogFeature::slotPlaylistTableRenamed(int playlistId, const QString& newName) {
     Q_UNUSED(newName);
-    //qDebug() << "slotPlaylistTableRenamed() playlistId:" << playlistId;
+    // qDebug() << "SetlogFeature::slotPlaylistTableRenamed() Id:" << playlistId;
     if (m_playlistDao.getHiddenType(playlistId) == PlaylistDAO::PLHT_SET_LOG) {
         updateChildModel(QSet<int>{playlistId});
     }
