@@ -101,11 +101,29 @@ BaseTrackPlayerImpl::BaseTrackPlayerImpl(
     // Track color of the current track
     m_pTrackColor = std::make_unique<ControlObject>(
             ConfigKey(getGroup(), "track_color"));
-
     m_pTrackColor->set(kNoTrackColor);
     m_pTrackColor->connectValueChangeRequest(
             this, &BaseTrackPlayerImpl::slotTrackColorChangeRequest);
 
+    m_pStarsUp = std::make_unique<ControlPushButton>(ConfigKey(getGroup(), "stars_up"));
+    connect(m_pStarsUp.get(),
+            &ControlObject::valueChanged,
+            this,
+            [this](double value) {
+                if (value > 0) {
+                    slotTrackRatingChangeRequestRelative(1);
+                }
+            });
+    m_pStarsDown = std::make_unique<ControlPushButton>(ConfigKey(getGroup(), "stars_down"));
+    connect(m_pStarsDown.get(),
+            &ControlObject::valueChanged,
+            this,
+            [this](double value) {
+                if (value > 0) {
+                    slotTrackRatingChangeRequestRelative(-1);
+                }
+            });
+
     // Deck cloning
     m_pCloneFromDeck = std::make_unique<ControlObject>(
             ConfigKey(getGroup(), "CloneFromDeck"),
@@ -327,6 +345,12 @@ void BaseTrackPlayerImpl::slotEjectTrack(double v) {
         return;
     }
 
+    // Don't allow eject while playing a track. We don't need to lock to
+    // call ControlObject::get() so this is fine.
+    if (m_pPlay->toBool()) {
+        return;
+    }
+
     mixxx::Duration elapsed = m_ejectTimer.restart();
 
     // Double-click always restores the last replaced track, i.e. un-eject the second
@@ -348,11 +372,6 @@ void BaseTrackPlayerImpl::slotEjectTrack(double v) {
         return;
     }
 
-    // Don't allow rejections while playing a track. We don't need to lock to
-    // call ControlObject::get() so this is fine.
-    if (m_pPlay->toBool()) {
-        return;
-    }
     m_pChannel->getEngineBuffer()->ejectTrack();
 }
 
@@ -706,31 +725,6 @@ void BaseTrackPlayerImpl::loadTrackFromGroup(const QString& group) {
     slotLoadTrack(pTrack, false);
 }
 
-void BaseTrackPlayerImpl::ensureStarControlsArePrepared() {
-    if (m_pStarsUp == nullptr) {
-        m_pStarsUp = std::make_unique<ControlPushButton>(ConfigKey(getGroup(), "stars_up"));
-        connect(m_pStarsUp.get(),
-                &ControlObject::valueChanged,
-                this,
-                [this](double value) {
-                    if (value > 0) {
-                        emit trackRatingChangeRequest(1);
-                    }
-                });
-    }
-    if (m_pStarsDown == nullptr) {
-        m_pStarsDown = std::make_unique<ControlPushButton>(ConfigKey(getGroup(), "stars_down"));
-        connect(m_pStarsDown.get(),
-                &ControlObject::valueChanged,
-                this,
-                [this](double value) {
-                    if (value > 0) {
-                        emit trackRatingChangeRequest(-1);
-                    }
-                });
-    }
-}
-
 void BaseTrackPlayerImpl::slotSetReplayGain(mixxx::ReplayGain replayGain) {
     // Do not change replay gain when track is playing because
     // this may lead to an unexpected volume change.
@@ -778,11 +772,22 @@ void BaseTrackPlayerImpl::slotTrackColorChangeRequest(double v) {
     m_pLoadedTrack->setColor(color);
 }
 
-void BaseTrackPlayerImpl::slotSetTrackRating(int rating) {
+void BaseTrackPlayerImpl::slotTrackRatingChangeRequest(int rating) {
     if (!m_pLoadedTrack) {
         return;
     }
-    m_pLoadedTrack->setRating(rating);
+    if (mixxx::TrackRecord::isValidRating(rating) &&
+            rating != m_pLoadedTrack->getRating()) {
+        m_pLoadedTrack->setRating(rating);
+        emit trackRatingChanged(rating);
+    }
+}
+
+void BaseTrackPlayerImpl::slotTrackRatingChangeRequestRelative(int change) {
+    if (!m_pLoadedTrack) {
+        return;
+    }
+    slotTrackRatingChangeRequest(m_pLoadedTrack->getRating() + change);
 }
 
 void BaseTrackPlayerImpl::slotPlayToggled(double value) {
