@@ -49,11 +49,6 @@ SoundManagerConfig::SoundManagerConfig(SoundManager* pSoundManager)
     m_configFile = QFileInfo(QDir(CmdlineArgs::Instance().getSettingsPath()).filePath(SOUNDMANAGERCONFIG_FILENAME));
 }
 
-SoundManagerConfig::~SoundManagerConfig() {
-    // don't write to disk here, it's SoundManager's responsibility
-    // to save its own configuration -- bkgood
-}
-
 /**
  * Read the SoundManagerConfig xml serialization at the predetermined
  * path
@@ -80,7 +75,8 @@ bool SoundManagerConfig::readFromDisk() {
     setForceNetworkClock(rootElement.attribute(xmlAttributeForceNetworkClock,
             "0").toUInt() != 0);
     setDeckCount(rootElement.attribute(xmlAttributeDeckCount,
-            QString(kDefaultDeckCount)).toUInt());
+                                    QString::number(kDefaultDeckCount))
+                         .toUInt());
     clearOutputs();
     clearInputs();
     QDomNodeList devElements(rootElement.elementsByTagName(xmlElementSoundDevice));
@@ -180,11 +176,11 @@ bool SoundManagerConfig::readFromDisk() {
                 continue;
             }
             AudioOutput out(AudioOutput::fromXML(outElement));
-            if (out.getType() == AudioPath::INVALID) {
+            if (out.getType() == AudioPathType::Invalid) {
                 continue;
             }
             bool dupe(false);
-            for (const AudioOutput& otherOut : qAsConst(m_outputs)) {
+            for (const AudioOutput& otherOut : std::as_const(m_outputs)) {
                 if (out == otherOut
                         && out.getChannelGroup() == otherOut.getChannelGroup()) {
                     dupe = true;
@@ -203,11 +199,11 @@ bool SoundManagerConfig::readFromDisk() {
                 continue;
             }
             AudioInput in(AudioInput::fromXML(inElement));
-            if (in.getType() == AudioPath::INVALID) {
+            if (in.getType() == AudioPathType::Invalid) {
                 continue;
             }
             bool dupe(false);
-            for (const AudioInput& otherIn : qAsConst(m_inputs)) {
+            for (const AudioInput& otherIn : std::as_const(m_inputs)) {
                 if (in == otherIn
                         && in.getChannelGroup() == otherIn.getChannelGroup()) {
                     dupe = true;
@@ -354,9 +350,9 @@ void SoundManagerConfig::setCorrectDeckCount(int configuredDeckCount) {
                 ++it) {
             const int index = it.value().getIndex();
             const AudioPathType type = it.value().getType();
-            if ((type == AudioInput::DECK ||
-                        type == AudioInput::VINYLCONTROL ||
-                        type == AudioInput::AUXILIARY) &&
+            if ((type == AudioPathType::Deck ||
+                        type == AudioPathType::VinylControl ||
+                        type == AudioPathType::Auxiliary) &&
                     index + 1 > minimum_deck_count) {
                 qDebug() << "Found an input connection above current deck count";
                 minimum_deck_count = index + 1;
@@ -367,7 +363,7 @@ void SoundManagerConfig::setCorrectDeckCount(int configuredDeckCount) {
                 ++it) {
             const int index = it.value().getIndex();
             const AudioPathType type = it.value().getType();
-            if (type == AudioOutput::DECK && index + 1 > minimum_deck_count) {
+            if (type == AudioPathType::Deck && index + 1 > minimum_deck_count) {
                 qDebug() << "Found an output connection above current deck count";
                 minimum_deck_count = index + 1;
             }
@@ -442,9 +438,9 @@ void SoundManagerConfig::addOutput(const SoundDeviceId &device, const AudioOutpu
 
 void SoundManagerConfig::addInput(const SoundDeviceId &device, const AudioInput &in) {
     m_inputs.insert(device, in);
-    if (in.getType() == AudioPath::MICROPHONE) {
+    if (in.getType() == AudioPathType::Microphone) {
         m_iNumMicInputs++;
-    } else if (in.getType() == AudioPath::RECORD_BROADCAST) {
+    } else if (in.getType() == AudioPathType::RecordBroadcast) {
         m_bExternalRecordBroadcastConnected = true;
     }
 }
@@ -476,13 +472,13 @@ bool SoundManagerConfig::hasExternalRecordBroadcast() {
 }
 
 /**
- * Loads default values for API, master output, sample rate and/or latency.
+ * Loads default values for API, main output, sample rate and/or latency.
  * @param soundManager pointer to SoundManager instance to load data from
  * @param flags Bitfield to determine which defaults to load, use something
  *              like SoundManagerConfig::API | SoundManagerConfig::DEVICES to
- *              load default API and master device.
+ *              load default API and main device.
  */
-void SoundManagerConfig::loadDefaults(SoundManager *soundManager, unsigned int flags) {
+void SoundManagerConfig::loadDefaults(SoundManager* soundManager, unsigned int flags) {
     if (flags & SoundManagerConfig::API) {
         QList<QString> apiList = soundManager->getHostAPIList();
         if (!apiList.isEmpty()) {
@@ -523,8 +519,8 @@ void SoundManagerConfig::loadDefaults(SoundManager *soundManager, unsigned int f
                 if (pDevice->getNumOutputChannels() < 2) {
                     continue;
                 }
-                AudioOutput masterOut(AudioPath::MASTER, 0, 2, 0);
-                addOutput(pDevice->getDeviceId(), masterOut);
+                auto mainOut = AudioOutput(AudioPathType::Main, 0, 2, 0);
+                addOutput(pDevice->getDeviceId(), mainOut);
                 defaultSampleRate = pDevice->getDefaultSampleRate();
                 break;
             }
