@@ -10,12 +10,12 @@
 
 #include "soundio/soundmanagerutil.h"
 
-class SoundDevice;
 class SoundManager;
 
 class SoundManagerConfig {
-public:
-    explicit SoundManagerConfig(SoundManager* pSoundManager);
+  public:
+    explicit SoundManagerConfig(
+            SoundManager* pSoundManager);
 
     enum Defaults {
         API = (1 << 0),
@@ -57,17 +57,14 @@ public:
     static const unsigned int kDefaultDeckCount;
     static const int kDefaultSyncBuffers;
 
-    SoundManagerConfig();
-    ~SoundManagerConfig();
-
     bool readFromDisk();
     bool writeToDisk() const;
     QString getAPI() const;
-    void setAPI(const QString &api);
+    void setAPI(const QString& api);
     bool checkAPI();
     unsigned int getSampleRate() const;
     void setSampleRate(unsigned int sampleRate);
-    bool checkSampleRate(const SoundManager &soundManager);
+    bool checkSampleRate(const SoundManager& soundManager);
 
     // Record the number of decks configured with this setup so they can
     // be created and configured.
@@ -83,16 +80,17 @@ public:
     void setSyncBuffers(unsigned int sampleRate);
     bool getForceNetworkClock() const;
     void setForceNetworkClock(bool force);
-    void addOutput(const SoundDeviceId &device, const AudioOutput &out);
-    void addInput(const SoundDeviceId &device, const AudioInput &in);
+    void addOutput(const SoundDeviceId& device, const AudioOutput& out);
+    void addInput(const SoundDeviceId& device, const AudioInput& in);
     QMultiHash<SoundDeviceId, AudioOutput> getOutputs() const;
     QMultiHash<SoundDeviceId, AudioInput> getInputs() const;
     void clearOutputs();
     void clearInputs();
     bool hasMicInputs();
     bool hasExternalRecordBroadcast();
-    void loadDefaults(SoundManager *soundManager, unsigned int flags);
-private:
+    void loadDefaults(SoundManager* soundManager, unsigned int flags);
+
+  private:
     QFileInfo m_configFile;
     QString m_api;
     // none of our sample rates are actually decimals, this avoids
