@@ -1,19 +1,20 @@
-#include <QScopedPointer>
-
 #include <gmock/gmock.h>
 
-#include "test/mixxxtest.h"
-#include "controllers/midi/midicontroller.h"
-#include "controllers/midi/midicontrollerpreset.h"
-#include "controllers/midi/midimessage.h"
-#include "control/controlpushbutton.h"
+#include <QScopedPointer>
+
 #include "control/controlpotmeter.h"
+#include "control/controlpushbutton.h"
+#include "controllers/midi/legacymidicontrollermapping.h"
+#include "controllers/midi/midicontroller.h"
+#include "controllers/midi/midimessage.h"
+#include "controllers/midi/midiutils.h"
+#include "test/mixxxtest.h"
 #include "util/time.h"
 
 class MockMidiController : public MidiController {
   public:
-    explicit MockMidiController(UserSettingsPointer pConfig)
-            : MidiController(pConfig) {
+    explicit MockMidiController()
+            : MidiController("test") {
     }
     ~MockMidiController() override { }
 
@@ -22,31 +23,35 @@ class MockMidiController : public MidiController {
     MOCK_METHOD3(sendShortMsg, void(unsigned char status,
                                     unsigned char byte1,
                                     unsigned char byte2));
-    MOCK_METHOD1(send, void(const QByteArray& data));
+    MOCK_METHOD1(sendBytes, void(const QByteArray& data));
     MOCK_CONST_METHOD0(isPolling, bool());
 };
 
 class MidiControllerTest : public MixxxTest {
   protected:
     void SetUp() override {
-        m_pController.reset(new MockMidiController(config()));
+        m_pController.reset(new MockMidiController());
+        m_pMapping = std::make_shared<LegacyMidiControllerMapping>();
     }
 
     void addMapping(const MidiInputMapping& mapping) {
-        m_preset.addInputMapping(mapping.key.key, mapping);
+        m_pMapping->addInputMapping(mapping.key.key, mapping);
     }
 
-    void loadPreset(const MidiControllerPreset& preset) {
-        m_pController->visit(&preset);
-    }
-
-    void receive(unsigned char status, unsigned char control,
-                 unsigned char value) {
+    void receivedShortMessage(unsigned char status, unsigned char control, unsigned char value) {
         // TODO(rryan): This test doesn't care about timestamps.
-        m_pController->receive(status, control, value, mixxx::Time::elapsed());
+        m_pController->receivedShortMessage(status, control, value, mixxx::Time::elapsed());
     }
 
-    MidiControllerPreset m_preset;
+    void receivedShortMessage(MidiOpCode opcode, uint8_t channel, uint8_t control, uint8_t value) {
+        ASSERT_TRUE((channel & 0xF) == channel);
+        receivedShortMessage(
+                MidiUtils::statusFromOpCodeAndChannel(opcode, channel),
+                control,
+                value);
+    }
+
+    std::shared_ptr<LegacyMidiControllerMapping> m_pMapping;
     QScopedPointer<MockMidiController> m_pController;
 };
 
@@ -59,22 +64,28 @@ TEST_F(MidiControllerTest, ReceiveMessage_PushButtonCO_PushOnOff) {
     unsigned char channel = 0x01;
     unsigned char control = 0x10;
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_ON | channel, control),
-                                MidiOptions(), key));
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_OFF | channel, control),
-                                MidiOptions(), key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOn, channel),
+                                        control),
+            MidiOptions(),
+            key));
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOff, channel),
+                                        control),
+            MidiOptions(),
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // Receive an on/off, sets the control on/off with each press.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 
     // Receive an on/off, sets the control on/off with each press.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 }
 
@@ -87,20 +98,23 @@ TEST_F(MidiControllerTest, ReceiveMessage_PushButtonCO_PushOnOn) {
     unsigned char channel = 0x01;
     unsigned char control = 0x10;
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_ON | channel, control),
-                                MidiOptions(), key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOn, channel),
+                                        control),
+            MidiOptions(),
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // Receive an on/off, sets the control on/off with each press.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
-    receive(MIDI_NOTE_ON | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 
     // Receive an on/off, sets the control on/off with each press.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
-    receive(MIDI_NOTE_ON | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 }
 
@@ -114,24 +128,30 @@ TEST_F(MidiControllerTest, ReceiveMessage_PushButtonCO_ToggleOnOff_ButtonMidiOpt
     unsigned char control = 0x10;
 
     MidiOptions options;
-    options.button = true;
+    options.setFlag(MidiOption::Button);
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_ON | channel, control),
-                                options, key));
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_OFF | channel, control),
-                                options, key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOn, channel),
+                                        control),
+            options,
+            key));
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOff, channel),
+                                        control),
+            options,
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // NOTE(rryan): This behavior is broken!
 
     // Toggle the switch on, sets the push button on.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
 
     // The push button is stuck down here!
 
     // Toggle the switch off, sets the push button off.
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 }
 
@@ -145,30 +165,36 @@ TEST_F(MidiControllerTest, ReceiveMessage_PushButtonCO_ToggleOnOff_SwitchMidiOpt
     unsigned char control = 0x10;
 
     MidiOptions options;
-    options.sw = true;
+    options.setFlag(MidiOption::Switch);
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_ON | channel, control),
-                                options, key));
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_OFF | channel, control),
-                                options, key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOn, channel),
+                                        control),
+            options,
+            key));
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOff, channel),
+                                        control),
+            options,
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // NOTE(rryan): This behavior is broken!
 
     // Toggle the switch on, sets the push button on.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
 
     // The push button is stuck down here!
 
     // Toggle the switch off, sets the push button on again.
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
     EXPECT_LT(0.0, cpb.get());
 
     // NOTE(rryan): What is supposed to happen in this case? It's an open
     // question I think. I think if you want to connect a switch MIDI control to
     // a push button CO then the switch should directly set the CO. After all,
-    // the preset author asked for the switch to be interpreted as a switch. If
+    // the mapping author asked for the switch to be interpreted as a switch. If
     // they want the switch to act like a push button, they should use the
     // button MIDI option.
     //
@@ -193,20 +219,24 @@ TEST_F(MidiControllerTest, ReceiveMessage_PushButtonCO_PushCC) {
     unsigned char channel = 0x01;
     unsigned char control = 0x10;
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_CC | channel, control),
-                                MidiOptions(), key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(
+            MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                            MidiOpCode::ControlChange, channel),
+                    control),
+            MidiOptions(),
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // Receive an on/off, sets the control on/off with each press.
-    receive(MIDI_CC | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
-    receive(MIDI_CC | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 
     // Receive an on/off, sets the control on/off with each press.
-    receive(MIDI_CC | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
-    receive(MIDI_CC | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 }
 
@@ -220,21 +250,27 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_PushOnOff) {
     unsigned char channel = 0x01;
     unsigned char control = 0x10;
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_ON | channel, control),
-                                MidiOptions(), key));
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_OFF | channel, control),
-                                MidiOptions(), key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOn, channel),
+                                        control),
+            MidiOptions(),
+            key));
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOff, channel),
+                                        control),
+            MidiOptions(),
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // Receive an on/off, toggles the control.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
 
     EXPECT_LT(0.0, cpb.get());
 
     // Receive an on/off, toggles the control.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
 
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 }
@@ -249,19 +285,22 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_PushOnOn) {
     unsigned char channel = 0x01;
     unsigned char control = 0x10;
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_ON | channel, control),
-                                MidiOptions(), key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOn, channel),
+                                        control),
+            MidiOptions(),
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // Receive an on/off, toggles the control.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
-    receive(MIDI_NOTE_ON | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x00);
 
     EXPECT_LT(0.0, cpb.get());
 
     // Receive an on/off, toggles the control.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
-    receive(MIDI_NOTE_ON | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x00);
 
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 }
@@ -277,13 +316,19 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_ToggleOnOff_ButtonMidiOption)
     unsigned char control = 0x10;
 
     MidiOptions options;
-    options.button = true;
+    options.setFlag(MidiOption::Button);
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_ON | channel, control),
-                                options, key));
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_OFF | channel, control),
-                                options, key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOn, channel),
+                                        control),
+            options,
+            key));
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOff, channel),
+                                        control),
+            options,
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // NOTE(rryan): If the intended behavior of the button MIDI option is to
     // make a toggle MIDI button act like a push button then this isn't
@@ -291,12 +336,12 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_ToggleOnOff_ButtonMidiOption)
 
     // Toggle the switch on, since it is interpreted as a button press it
     // toggles the button on.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
 
     // Toggle the switch off, since it is interpreted as a button release it
     // does nothing to the toggle button.
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
     EXPECT_LT(0.0, cpb.get());
 }
 
@@ -311,13 +356,19 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_ToggleOnOff_SwitchMidiOption)
     unsigned char control = 0x10;
 
     MidiOptions options;
-    options.sw = true;
+    options.setFlag(MidiOption::Switch);
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_ON | channel, control),
-                                options, key));
-    addMapping(MidiInputMapping(MidiKey(MIDI_NOTE_OFF | channel, control),
-                                options, key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOn, channel),
+                                        control),
+            options,
+            key));
+    addMapping(MidiInputMapping(MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                                                MidiOpCode::NoteOff, channel),
+                                        control),
+            options,
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // NOTE(rryan): If the intended behavior of switch MIDI option is to make a
     // toggle MIDI button act like a toggle button then this isn't working. The
@@ -326,12 +377,12 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_ToggleOnOff_SwitchMidiOption)
 
     // Toggle the switch on, since it is interpreted as a button press it
     // toggles the control on.
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_LT(0.0, cpb.get());
 
     // Toggle the switch off, since it is interpreted as a button press it
     // toggles the control off.
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 
     // Meanwhile, the GUI toggles the control on again.
@@ -341,12 +392,12 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_ToggleOnOff_SwitchMidiOption)
 
     // Toggle the switch on, since it is interpreted as a button press it
     // toggles the control off (since it was on).
-    receive(MIDI_NOTE_ON | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::NoteOn, channel, control, 0x7F);
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 
     // Toggle the switch off, since it is interpreted as a button press it
     // toggles the control on (since it was off).
-    receive(MIDI_NOTE_OFF | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::NoteOff, channel, control, 0x00);
     EXPECT_LT(0.0, cpb.get());
 }
 
@@ -360,19 +411,23 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_PushCC) {
     unsigned char channel = 0x01;
     unsigned char control = 0x10;
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_CC | channel, control),
-                                MidiOptions(), key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(
+            MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                            MidiOpCode::ControlChange, channel),
+                    control),
+            MidiOptions(),
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // Receive an on/off, toggles the control.
-    receive(MIDI_CC | channel, control, 0x7F);
-    receive(MIDI_CC | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x00);
 
     EXPECT_LT(0.0, cpb.get());
 
     // Receive an on/off, toggles the control.
-    receive(MIDI_CC | channel, control, 0x7F);
-    receive(MIDI_CC | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x00);
 
     EXPECT_DOUBLE_EQ(0.0, cpb.get());
 }
@@ -380,37 +435,41 @@ TEST_F(MidiControllerTest, ReceiveMessage_ToggleCO_PushCC) {
 TEST_F(MidiControllerTest, ReceiveMessage_PotMeterCO_7BitCC) {
     ConfigKey key("[Channel1]", "playposition");
 
-    const double kMinValue = -1234.5;
-    const double kMaxValue = 678.9;
-    const double kMiddleValue = (kMinValue + kMaxValue) * 0.5;
+    constexpr double kMinValue = -1234.5;
+    constexpr double kMaxValue = 678.9;
+    constexpr double kMiddleValue = (kMinValue + kMaxValue) * 0.5;
     ControlPotmeter potmeter(key, kMinValue, kMaxValue);
 
     unsigned char channel = 0x01;
     unsigned char control = 0x10;
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_CC | channel, control),
-                                MidiOptions(), key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(
+            MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                            MidiOpCode::ControlChange, channel),
+                    control),
+            MidiOptions(),
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // Receive a 0, MIDI parameter should map to the min value.
-    receive(MIDI_CC | channel, control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x00);
     EXPECT_DOUBLE_EQ(kMinValue, potmeter.get());
 
     // Receive a 0x7F, MIDI parameter should map to the potmeter max value.
-    receive(MIDI_CC | channel, control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x7F);
     EXPECT_DOUBLE_EQ(kMaxValue, potmeter.get());
 
     // Receive a 0x40, MIDI parameter should map to the potmeter middle value.
-    receive(MIDI_CC | channel, control, 0x40);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, control, 0x40);
     EXPECT_DOUBLE_EQ(kMiddleValue, potmeter.get());
 }
 
 TEST_F(MidiControllerTest, ReceiveMessage_PotMeterCO_14BitCC) {
     ConfigKey key("[Channel1]", "playposition");
 
-    const double kMinValue = -1234.5;
-    const double kMaxValue = 678.9;
-    const double kMiddleValue = (kMinValue + kMaxValue) * 0.5;
+    constexpr double kMinValue = -1234.5;
+    constexpr double kMaxValue = 678.9;
+    constexpr double kMiddleValue = (kMinValue + kMaxValue) * 0.5;
     ControlPotmeter potmeter(key, kMinValue, kMaxValue);
     potmeter.set(0);
 
@@ -419,16 +478,24 @@ TEST_F(MidiControllerTest, ReceiveMessage_PotMeterCO_14BitCC) {
     unsigned char msb_control = 0x11;
 
     MidiOptions lsb;
-    lsb.fourteen_bit_lsb = true;
+    lsb.setFlag(MidiOption::FourteenBitLSB);
 
     MidiOptions msb;
-    msb.fourteen_bit_msb = true;
+    msb.setFlag(MidiOption::FourteenBitMSB);
 
-    addMapping(MidiInputMapping(MidiKey(MIDI_CC | channel, lsb_control),
-                                lsb, key));
-    addMapping(MidiInputMapping(MidiKey(MIDI_CC | channel, msb_control),
-                                msb, key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(
+            MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                            MidiOpCode::ControlChange, channel),
+                    lsb_control),
+            lsb,
+            key));
+    addMapping(MidiInputMapping(
+            MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                            MidiOpCode::ControlChange, channel),
+                    msb_control),
+            msb,
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // If kMinValue or kMaxValue are such that the middle value is 0 then the
     // set(0) commands below allow us to hide failures.
@@ -436,40 +503,40 @@ TEST_F(MidiControllerTest, ReceiveMessage_PotMeterCO_14BitCC) {
 
     // Receive a 0x0000 (lsb-first), MIDI parameter should map to the min value.
     potmeter.set(0);
-    receive(MIDI_CC | channel, lsb_control, 0x00);
-    receive(MIDI_CC | channel, msb_control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, lsb_control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, msb_control, 0x00);
     EXPECT_DOUBLE_EQ(kMinValue, potmeter.get());
 
     // Receive a 0x0000 (msb-first), MIDI parameter should map to the min value.
     potmeter.set(0);
-    receive(MIDI_CC | channel, msb_control, 0x00);
-    receive(MIDI_CC | channel, lsb_control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, msb_control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, lsb_control, 0x00);
     EXPECT_DOUBLE_EQ(kMinValue, potmeter.get());
 
     // Receive a 0x3FFF (lsb-first), MIDI parameter should map to the max value.
     potmeter.set(0);
-    receive(MIDI_CC | channel, lsb_control, 0x7F);
-    receive(MIDI_CC | channel, msb_control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, lsb_control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, msb_control, 0x7F);
     EXPECT_DOUBLE_EQ(kMaxValue, potmeter.get());
 
     // Receive a 0x3FFF (msb-first), MIDI parameter should map to the max value.
     potmeter.set(0);
-    receive(MIDI_CC | channel, msb_control, 0x7F);
-    receive(MIDI_CC | channel, lsb_control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, msb_control, 0x7F);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, lsb_control, 0x7F);
     EXPECT_DOUBLE_EQ(kMaxValue, potmeter.get());
 
     // Receive a 0x2000 (lsb-first), MIDI parameter should map to the middle
     // value.
     potmeter.set(0);
-    receive(MIDI_CC | channel, lsb_control, 0x00);
-    receive(MIDI_CC | channel, msb_control, 0x40);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, lsb_control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, msb_control, 0x40);
     EXPECT_DOUBLE_EQ(kMiddleValue, potmeter.get());
 
     // Receive a 0x2000 (msb-first), MIDI parameter should map to the middle
     // value.
     potmeter.set(0);
-    receive(MIDI_CC | channel, msb_control, 0x40);
-    receive(MIDI_CC | channel, lsb_control, 0x00);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, msb_control, 0x40);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, lsb_control, 0x00);
     EXPECT_DOUBLE_EQ(kMiddleValue, potmeter.get());
 
     // Check the 14-bit resolution is actually present. Receive a 0x2001
@@ -477,8 +544,8 @@ TEST_F(MidiControllerTest, ReceiveMessage_PotMeterCO_14BitCC) {
     // amount. Scaling is not quite linear for MIDI parameters so just check
     // that incrementing the LSB by 1 is greater than the middle value.
     potmeter.set(0);
-    receive(MIDI_CC | channel, msb_control, 0x40);
-    receive(MIDI_CC | channel, lsb_control, 0x01);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, msb_control, 0x40);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, lsb_control, 0x01);
     EXPECT_LT(kMiddleValue, potmeter.get());
 
     // Check the 14-bit resolution is actually present. Receive a 0x2001
@@ -486,42 +553,46 @@ TEST_F(MidiControllerTest, ReceiveMessage_PotMeterCO_14BitCC) {
     // amount. Scaling is not quite linear for MIDI parameters so just check
     // that incrementing the LSB by 1 is greater than the middle value.
     potmeter.set(0);
-    receive(MIDI_CC | channel, lsb_control, 0x01);
-    receive(MIDI_CC | channel, msb_control, 0x40);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, lsb_control, 0x01);
+    receivedShortMessage(MidiOpCode::ControlChange, channel, msb_control, 0x40);
     EXPECT_LT(kMiddleValue, potmeter.get());
 }
 
 TEST_F(MidiControllerTest, ReceiveMessage_PotMeterCO_14BitPitchBend) {
     ConfigKey key("[Channel1]", "rate");
 
-    const double kMinValue = -1234.5;
-    const double kMaxValue = 678.9;
-    const double kMiddleValue = (kMinValue + kMaxValue) * 0.5;
+    constexpr double kMinValue = -1234.5;
+    constexpr double kMaxValue = 678.9;
+    constexpr double kMiddleValue = (kMinValue + kMaxValue) * 0.5;
     ControlPotmeter potmeter(key, kMinValue, kMaxValue);
     unsigned char channel = 0x01;
 
     // The control is ignored in mappings for messages where the control is part
     // of the payload.
-    addMapping(MidiInputMapping(MidiKey(MIDI_PITCH_BEND | channel, 0xFF),
-                                MidiOptions(), key));
-    loadPreset(m_preset);
+    addMapping(MidiInputMapping(
+            MidiKey(MidiUtils::statusFromOpCodeAndChannel(
+                            MidiOpCode::PitchBendChange, channel),
+                    0xFF),
+            MidiOptions(),
+            key));
+    m_pController->setMapping(m_pMapping->clone());
 
     // Receive a 0x0000, MIDI parameter should map to the min value.
-    receive(MIDI_PITCH_BEND | channel, 0x00, 0x00);
+    receivedShortMessage(MidiOpCode::PitchBendChange, channel, 0x00, 0x00);
     EXPECT_DOUBLE_EQ(kMinValue, potmeter.get());
 
     // Receive a 0x3FFF, MIDI parameter should map to the potmeter max value.
-    receive(MIDI_PITCH_BEND | channel, 0x7F, 0x7F);
+    receivedShortMessage(MidiOpCode::PitchBendChange, channel, 0x7F, 0x7F);
     EXPECT_DOUBLE_EQ(kMaxValue, potmeter.get());
 
     // Receive a 0x2000, MIDI parameter should map to the potmeter middle value.
-    receive(MIDI_PITCH_BEND | channel, 0x00, 0x40);
+    receivedShortMessage(MidiOpCode::PitchBendChange, channel, 0x00, 0x40);
     EXPECT_DOUBLE_EQ(kMiddleValue, potmeter.get());
 
     // Check the 14-bit resolution is actually present. Receive a 0x2001, MIDI
     // parameter should map to the middle value plus a tiny amount. Scaling is
     // not quite linear for MIDI parameters so just check that incrementing the
     // LSB by 1 is greater than the middle value.
-    receive(MIDI_PITCH_BEND | channel, 0x01, 0x40);
+    receivedShortMessage(MidiOpCode::PitchBendChange, channel, 0x01, 0x40);
     EXPECT_LT(kMiddleValue, potmeter.get());
 }
