@@ -4,11 +4,6 @@
 #include "sources/metadatasourcetaglib.h"
 #include "test/mixxxtest.h"
 
-namespace {
-
-const QDir kTestDir = QDir::current().absoluteFilePath("src/test/id3-test-data");
-
-} // anonymous namespace
 
 class TagLibTest : public testing::Test {
 };
@@ -21,7 +16,9 @@ TEST_F(TagLibTest, WriteID3v2Tag) {
     const QString tmpFileName = tempDir.filePath("no_id3v1_mp3");
 
     // Create the temporary file by copying an existing file
-    mixxxtest::copyFile(kTestDir.absoluteFilePath("empty.mp3"), tmpFileName);
+    mixxxtest::copyFile(
+            MixxxTest::getOrInitTestDir().filePath(QStringLiteral("id3-test-data/empty.mp3")),
+            tmpFileName);
 
     // Verify that the file has no tags
     {
@@ -36,7 +33,7 @@ TEST_F(TagLibTest, WriteID3v2Tag) {
 
     // Write metadata -> only an ID3v2 tag should be added
     mixxx::TrackMetadata trackMetadata;
-    trackMetadata.refTrackInfo().setTitle("title");
+    trackMetadata.refTrackInfo().setTitle(QStringLiteral("title"));
     const auto exported =
             mixxx::MetadataSourceTagLib(
                     tmpFileName, mixxx::taglib::FileType::MP3)
@@ -56,7 +53,7 @@ TEST_F(TagLibTest, WriteID3v2Tag) {
     qDebug() << "Updating track title";
 
     // Write metadata again -> only the ID3v2 tag should be modified
-    trackMetadata.refTrackInfo().setTitle("title2");
+    trackMetadata.refTrackInfo().setTitle(QStringLiteral("title2"));
     const auto exported2 =
             mixxx::MetadataSourceTagLib(
                     tmpFileName, mixxx::taglib::FileType::MP3)
