@@ -17,6 +17,28 @@
 #include "sources/soundsourceproxy.h"
 #include "util/assert.h"
 
+namespace {
+
+bool calcUseColorsAuto() {
+    // see https://no-color.org/
+    if (QProcessEnvironment::systemEnvironment().contains(QLatin1String("NO_COLOR"))) {
+        return false;
+    } else {
+#ifndef __WINDOWS__
+        if (isatty(fileno(stderr))) {
+            return true;
+        }
+#else
+        if (_isatty(_fileno(stderr))) {
+            return true;
+        }
+#endif
+    }
+    return false;
+}
+
+} // namespace
+
 CmdlineArgs::CmdlineArgs()
         : m_startInFullscreen(false), // Initialize vars
           m_controllerDebug(false),
@@ -28,7 +50,7 @@ CmdlineArgs::CmdlineArgs()
           m_debugAssertBreak(false),
           m_settingsPathSet(false),
           m_scaleFactor(1.0),
-          m_useColors(false),
+          m_useColors(calcUseColorsAuto()),
           m_parseForUserFeedbackRequired(false),
           m_logLevel(mixxx::kLogLevelDefault),
           m_logFlushLevel(mixxx::kLogFlushLevelDefault),
@@ -388,26 +410,11 @@ bool CmdlineArgs::parse(const QStringList& arguments, CmdlineArgs::ParseMode mod
     }
 
     // set colors
-    if (parser.value(color).compare(QLatin1String("auto"), Qt::CaseInsensitive) == 0) {
-        // see https://no-color.org/
-        if (QProcessEnvironment::systemEnvironment().contains(QLatin1String("NO_COLOR"))) {
-            m_useColors = false;
-        } else {
-#ifndef __WINDOWS__
-            if (isatty(fileno(stderr))) {
-                m_useColors = true;
-            }
-#else
-            if (_isatty(_fileno(stderr))) {
-                m_useColors = true;
-            }
-#endif
-        }
-    } else if (parser.value(color).compare(QLatin1String("always"), Qt::CaseInsensitive) == 0) {
+    if (parser.value(color).compare(QLatin1String("always"), Qt::CaseInsensitive) == 0) {
         m_useColors = true;
     } else if (parser.value(color).compare(QLatin1String("never"), Qt::CaseInsensitive) == 0) {
         m_useColors = false;
-    } else {
+    } else if (parser.value(color).compare(QLatin1String("auto"), Qt::CaseInsensitive) != 0) {
         fputs("Unknown argument for for color.\n", stdout);
     }
 
