@@ -1,13 +1,13 @@
 #pragma once
 
-#include <QWidget>
+#include "waveform/widgets/nonglwaveformwidgetabstract.h"
 
-#include "waveform/widgets/waveformwidgetabstract.h"
+class QWidget;
 
 // This class can be used as a template file to create new WaveformWidgets it
 // contain minimal set of method to re-implement
 
-class EmptyWaveformWidget : public QWidget, public WaveformWidgetAbstract {
+class EmptyWaveformWidget : public NonGLWaveformWidgetAbstract {
     Q_OBJECT
   public:
     virtual ~EmptyWaveformWidget();
@@ -18,7 +18,9 @@ class EmptyWaveformWidget : public QWidget, public WaveformWidgetAbstract {
     static inline bool useOpenGl() { return false; }
     static inline bool useOpenGles() { return false; }
     static inline bool useOpenGLShaders() { return false; }
-    static inline bool developerOnly() { return false; }
+    static inline WaveformWidgetCategory category() {
+        return WaveformWidgetCategory::Default;
+    }
 
   protected:
     virtual void castToQWidget();
