@@ -1,18 +1,17 @@
 #pragma once
 
-#include <QPixmap>
-#include <QHash>
-#include <QSharedPointer>
-#include <QSvgRenderer>
 #include <QImage>
 #include <QScopedPointer>
-#include <QPainter>
 #include <QRectF>
 #include <QString>
 
 #include "skin/legacy/imgsource.h"
 #include "skin/legacy/pixmapsource.h"
 
+class QPainter;
+class QPixmap;
+class QSvgRenderer;
+
 // Wrapper around QImage and QSvgRenderer to support rendering SVG images in
 // high fidelity.
 class Paintable {
