@@ -1,12 +1,12 @@
 #include "widget/wcolorpicker.h"
 
 #include <QColorDialog>
-#include <QMapIterator>
+#include <QGridLayout>
 #include <QPushButton>
 #include <QStyle>
+#include <QStyleFactory>
 
 #include "moc_wcolorpicker.cpp"
-#include "util/color/color.h"
 #include "util/parented_ptr.h"
 
 namespace {
@@ -44,7 +44,6 @@ WColorPicker::WColorPicker(Options options, const ColorPalette& palette, QWidget
           m_pNoColorButton(nullptr),
           m_pCustomColorButton(nullptr) {
     QGridLayout* pLayout = new QGridLayout();
-    pLayout->setMargin(0);
     pLayout->setContentsMargins(0, 0, 0, 0);
 
     pLayout->setSizeConstraint(QLayout::SetFixedSize);
@@ -214,7 +213,7 @@ void WColorPicker::addCustomColorButton(QGridLayout* pLayout, int row, int colum
     pLayout->addWidget(pButton, row, column);
 }
 
-void WColorPicker::setColorButtonChecked(mixxx::RgbColor::optional_t color, bool checked) {
+void WColorPicker::setColorButtonChecked(const mixxx::RgbColor::optional_t& color, bool checked) {
     // Unset currently selected color
     QPushButton* pButton = nullptr;
     if (color) {
@@ -235,7 +234,6 @@ void WColorPicker::setColorButtonChecked(mixxx::RgbColor::optional_t color, bool
         pButton->setIcon(QIcon(checked ? ":/images/ic_checkmark.svg" : ""));
     }
     // This is needed to re-apply skin styles (e.g. to show/hide a checkmark icon)
-    pButton->style()->unpolish(pButton);
     pButton->style()->polish(pButton);
 }
 
@@ -243,7 +241,7 @@ void WColorPicker::resetSelectedColor() {
     setColorButtonChecked(m_selectedColor, false);
 }
 
-void WColorPicker::setSelectedColor(mixxx::RgbColor::optional_t color) {
+void WColorPicker::setSelectedColor(const mixxx::RgbColor::optional_t& color) {
     resetSelectedColor();
 
     m_selectedColor = color;
@@ -261,6 +259,6 @@ void WColorPicker::setColorPalette(const ColorPalette& palette) {
     addColorButtons();
 }
 
-void WColorPicker::slotColorPicked(mixxx::RgbColor::optional_t color) {
+void WColorPicker::slotColorPicked(const mixxx::RgbColor::optional_t& color) {
     setSelectedColor(color);
 }
