@@ -1,12 +1,7 @@
 #pragma once
 
-#include <QWidget>
-#include <QLabel>
-#include <QDomNode>
-
-#include "effects/effectchainslot.h"
+#include "effects/defs.h"
 #include "widget/wlabel.h"
-#include "skin/legacy/skincontext.h"
 
 class EffectsManager;
 
@@ -18,12 +13,12 @@ class WEffectChain : public WLabel {
     void setup(const QDomNode& node, const SkinContext& context) override;
 
   private slots:
-    void chainUpdated();
+    void chainPresetChanged(const QString& newName);
 
   private:
     // Set the EffectChain that should be monitored by this WEffectChain
-    void setEffectChainSlot(EffectChainSlotPointer pEffectChainSlot);
+    void setEffectChain(EffectChainPointer pEffectChain);
 
     EffectsManager* m_pEffectsManager;
-    EffectChainSlotPointer m_pEffectChainSlot;
+    EffectChainPointer m_pEffectChain;
 };
